﻿local EJ_GetEncounterInfo, GetDifficultyInfo, EJ_GetEncounterInfoByIndex = EJ_GetEncounterInfo, GetDifficultyInfo, EJ_GetEncounterInfoByIndex

local L = LootsturmLightLocalization

local matchStart = 1
local prevEnd = 0
local tmp = {}
local rollMassagePermutation = {}
for i = 1, 4 do
	matchStart, matchEnd, position = RANDOM_ROLL_RESULT:find("%%(%d?)%$?[sd]", prevEnd + 1)
	rollMassagePermutation[i] = tonumber(position) or i
	tmp[i] = RANDOM_ROLL_RESULT:sub(prevEnd + 1, matchStart - 1):gsub("([%^%$%(%)%%%.%[%]%*%+%-%?])", "%%%1")
	prevEnd = matchEnd
end
tmp[5] = RANDOM_ROLL_RESULT:sub(prevEnd + 1, RANDOM_ROLL_RESULT:len()):gsub("([%^%$%(%)%%%.%[%]%*%+%-%?])", "%%%1")
local rollMessageRegex = ""
for i = 1, 4 do
	rollMessageRegex = rollMessageRegex .. tmp[i]
	if rollMassagePermutation[i] == 1 then
		rollMessageRegex = rollMessageRegex .. "([^%s]+)"
	else
		rollMessageRegex = rollMessageRegex .. "(%d+)"
	end
end
rollMessageRegex = rollMessageRegex .. tmp[5]
local function parseRollMessage(message)
	local out = {message:match(rollMessageRegex)}
	return 
		out[rollMassagePermutation[1]], 
		tonumber(out[rollMassagePermutation[2]]), 
		tonumber(out[rollMassagePermutation[3]]), 
		tonumber(out[rollMassagePermutation[4]])
end

local interface = LootsturmLightInterface
local ADDON_MESSAGE_IDENTIFIER = "LSL_MESSAGE"
local data
local core = {}

local SOURCE = {
	PROGRESS = "progress",
	FARM     = "farm",
	SECOND   = "second",
	OTHER    = "other"
}

local context = {
	active = false,
	lootmaster = false,
	lootTable = {},
	receivedCache = {},
	encounterId = 0
}

local blizzardAPI = {
	GetLootThreshold       = function(self, ...) return GetLootThreshold(...) end,
	GetNumLootItems        = function(self, ...) return GetNumLootItems(...) end,
	GetLootSlotInfo        = function(self, ...) return GetLootSlotInfo(...) end,
	GetLootSlotLink        = function(self, ...) return GetLootSlotLink(...) end,
	GetMasterLootCandidate = function(self, ...) return GetMasterLootCandidate(...) end,
	GetLootMethod          = function(self, ...) return GetLootMethod(...) end,
	GetInstanceInfo        = function(self, ...) return GetInstanceInfo(...) end
}

local simulatorAPI = {
	loot = {},
	candidates = {},
	GetLootThreshold       = function(self) 
		return 4
	end,
	GetNumLootItems        = function(self)
		return #self.loot
	end,
	GetLootSlotInfo        = function(self, slot)
		return self.loot[slot].texture, self.loot[slot].itemText, 1, 4
	end,
	GetLootSlotLink        = function(self, slot)
		return self.loot[slot].itemLink
	end,
	GetMasterLootCandidate = function(self, slot, i)
		return self.candidates[i]
	end,
	GetLootMethod          = function(self)
		return "master", 0, nil
	end,
	GetInstanceInfo        = function(self)
		-- 14 - nh, 15 - hc, 16 - my  -> GetDifficultyInfo
		return "Dummy Instance", "raid", 14, GetDifficultyInfo(14), 20, 0, false, 0, 20
	end
}

local API = blizzardAPI

SLASH_LOOTSTURMLIGHT1, SLASH_LOOTSTURMLIGHT2 = '/lsl', '/lootsturmlight'
local messenger 
messenger = {
	add = function(self, prefix, message, channel, recipient) 
		self.nextMsg = {
			["prefix"] = prefix,
			["message"] = message,
			["channel"] = channel,
			["recipient"] = recipient,
			["nextMsg"] = self.nextMsg
		}
		if not self.scheduled then
			self.scheduled = true
			C_Timer.After(0.1, function() messenger:send(30) end)
		end
	end,
	send = function(self, num)
		local unschedule = false
		for count = 1,num do
			local m = self.nextMsg
			if m then
				self.nextMsg = self.nextMsg.nextMsg
				SendAddonMessage(m.prefix, m.message, m.channel, m.recipient)
				m = nil
			else
				unschedule = true
				break
			end
		end
		if unschedule then
			self.scheduled = false
		else
			C_Timer.After(1, function() messenger:send(30) end)
			self.scheduled = true
		end
	end
}

local function encodeAddonMessage(...)
	local msg = ""
	local sep = ""
	local n = select("#",...)
	local value, strValue
	for i = 1,n do
		value = select(i, ...)
		if type(value) == "nil" then
			strValue = "nil"
		elseif type(value) == "boolean" then
			strValue = (value and "true") or "false"
		elseif type(value) == "number" then
			strValue = tostring(value)
		else
			strValue = value
		end
		msg = msg .. sep .. strValue
		sep = "\n"
	end
	return msg
end

local function decodeAddonMessage(msg)
	local tbl = {}
	local value
	local index = 1
	for v in string.gmatch((msg or "") .. "\n", "([^\n]*)\n") do 
		value = v
		if value == "nil" then
			value = nil
		elseif value == "true" then
			value = true
		elseif value == "false" then
			value = false
		end
		tbl[index] = value
		index = index + 1
	end
	return tbl
end

local function encounterEnd(fakeEncounterId, encounterName, difficultyId, raidSize, endStatus)
	local encounterId = 0
	for id, data in pairs(data:getBossData()) do
		if EJ_GetEncounterInfo(id) == encounterName then
			encounterId = id
			break
		end
	end
	context.encounterId = encounterId
end

local function lootOpened()
	if not context.lootmaster then return end
	context.receivedCache = {}
	context.lootFrameClampedToScreen = LootFrame:IsClampedToScreen()
	_, _, context.difficultyId = API:GetInstanceInfo()
	context.isProgressBoss = data:isProgressBoss(context.encounterId, context.difficultyId)
	local threshold = API:GetLootThreshold()
	local numItems = API:GetNumLootItems()
	context.lootTable = {}
	local masterLootRequired = false
	for slot = 1,numItems do
		local texture, item, quantity, quality, locked = API:GetLootSlotInfo(slot)	
		local link = API:GetLootSlotLink(slot)
		local allCandidates = {}  
		for i = 1,40 do
			local name = API:GetMasterLootCandidate(slot, i)
			if name then
				allCandidates[data:getCanonicalPlayerName(name)] = true
			end
		end
		local itemId = link and data:decodeItemLink(link).id
		context.lootTable[slot] = {
			item = link,
			id = itemId,
			itemText = item,
			texture = texture,
			quantity = quantity,
			quality = quality,
			looted = false,
			eligibleCandidates = allCandidates,
			candidates = {}
		}
		masterLootRequired = masterLootRequired or quality >= threshold
	end
	context.masterLootRequired = masterLootRequired
	if context.masterLootRequired then
		SendChatMessage(L.LOOT .. ":", "RAID")
		for slot = 1,numItems do
			local item = context.lootTable[slot]
			SendChatMessage(("%d. %s"):format(slot, item.item or item.itemText), "RAID")
			messenger:add(ADDON_MESSAGE_IDENTIFIER, encodeAddonMessage("LOOT_OPENED", slot, numItems, item.item, item.itemText, item.texture, item.quantity, item.quality, context.encounterId, context.isProgressBoss), "RAID")
		end
		SendChatMessage(L.LOOT_RULE_INFO, "RAID")
		context.lootSlot = nil
		LootFrame:SetClampedToScreen(false)
		LootFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", -LootFrame:GetWidth(), LootFrame:GetHeight())
		local encounterList = { {caption = L.BOSS_UNKNOWN, value = 0} }
		for encounterId, encounter in pairs(data:getBossData()) do
			encounterList[#encounterList+1] = { caption = EJ_GetEncounterInfo(encounterId), value = encounterId }
		end
		interface:showLoot(context.encounterId, context.isProgressBoss, encounterList)
	end
end

local function lootClosed()
	if context.lootmaster then
		LootFrame:SetClampedToScreen(context.lootFrameClampedToScreen)
		if context.masterLootRequired then
			messenger:add(ADDON_MESSAGE_IDENTIFIER, encodeAddonMessage("LOOT_CLOSED"), "RAID")
		end
		for index, item in pairs(context.lootTable) do
			if item.looted and item.item and item.recipient then
				local dbPlayer = data:getPlayer(item.recipient)
				if not dbPlayer then
					data:addPlayer(item.recipient)
				end
				local c
				for index, candidate in pairs(item.candidates) do
					if candidate.player == item.recipient then
						c = candidate
						break
					end
				end
				local source = SOURCE.OTHER
				if c then
					if c.need then
						if context.isProgressBoss then
							source = SOURCE.PROGRESS
						else
							source = SOURCE.FARM
						end
					elseif c.greed then
						source = SOURCE.SECOND
					end
				end
				data:addReceivedItem(item.recipient, item.item, source, context.encounterId, context.difficultyId)
				print(("Added: %s -> %s"):format(item.item, item.recipient))
			end
		end
	end
	context.lootItemsReceived = 0
	context.lootTable = {}
	context.lootSlot = nil
	interface:showLoot()	
end

local function lootSlotCleared(slotId)
	if context.lootmaster and context.masterLootRequired then
		messenger:add(ADDON_MESSAGE_IDENTIFIER, encodeAddonMessage("LOOT_SLOT_CLEARED", slotId), "RAID")
	end
	if context.lootTable[slotId] then
		context.lootTable[slotId].looted = true
		if context.lootSlot == slotId then
			context.lootSlot = nil
		end
		interface:setLooted(slotId)
	end
end

local function chatMsgSystem(msg)
	if not context.lootmaster then return end
	if context.lootSlot then
		local player, roll, from, to = parseRollMessage(msg)
		roll = tonumber(roll)
		from = tonumber(from)
		to   = tonumber(to)
		if player and roll and from and to then
			local item = context.lootTable[context.lootSlot]
			if item and not item.looted and item.item then
				player = data:getCanonicalPlayerName(player) -- first guess
				if not item.eligibleCandidates[player] then
					local name = data:getPlayerNameAndRealm(player)
					local found = false
					for fullName, _ in pairs(item.eligibleCandidates) do
						if name == fullName:match("^[^%-]+") then
							found = true
							player = fullName
							break
						end
					end
					if not found then
						return
					end
				end
				local rollType
				if to == 100 then
					rollType = "need"
				elseif to == 50 then
					rollType = "greed"
				elseif to == 10 then
					rollType = "disenchant"
				end
				if from == 1 and rollType then
					local c, i
					local receivedProg, receivedFarm = core:getNumReceivedItems(player)
					for index, candidate in pairs(item.candidates) do
						if candidate.player == player then
							c = candidate
							i = index
							break
						end
					end
					if not c then
						local n, r = data:getPlayerNameAndRealm(player)
						c = {
							player = player,
							name = n,
							realm = r,
							[rollType] = roll,
							isRaidMember = data:isRaidMember(player)
						}
						item.candidates[#item.candidates+1] = c
						i = #item.candidates
						messenger:add(ADDON_MESSAGE_IDENTIFIER, encodeAddonMessage(
							"ROLL", 
							context.lootSlot, 
							i, 
							player,
							n,
							r,
							rollType,
							roll,
							false,
							receivedProg,
							receivedFarm,
							c.isRaidMember
						), "RAID")
						interface.updateLootCandidates()
					else	
						if not c[rollType] or c[rollType] == 0 then
							c[rollType] = roll
							c.passed = false
							messenger:add(ADDON_MESSAGE_IDENTIFIER, encodeAddonMessage(
								"ROLL", 
								context.lootSlot, 
								i, 
								c.player,
								c.name,
								c.realm,
								rollType,
								roll,
								false,
								receivedProg,
								receivedFarm,
								c.isRaidMember
							), "RAID")
							interface.updateLootCandidates()
						end
					end
				end
			end
		end
	end
end

local function partyLootMethodChanged()
	local method, partyMaster, raidMaster = API:GetLootMethod()
	if method == "master" then
		context.active = true
		context.lootmaster = partyMaster == 0
		context.mainWindow:RegisterEvent("CHAT_MSG_ADDON")
		if context.lootmaster then
			context.mainWindow:RegisterEvent("LOOT_OPENED")
			context.mainWindow:RegisterEvent("LOOT_CLOSED")
			context.mainWindow:RegisterEvent("LOOT_SLOT_CLEARED")
			context.mainWindow:RegisterEvent("CHAT_MSG_SYSTEM")
			context.mainWindow:RegisterEvent("ENCOUNTER_END")
		else
			context.mainWindow:UnregisterEvent("LOOT_OPENED")
			context.mainWindow:UnregisterEvent("LOOT_CLOSED")
			context.mainWindow:UnregisterEvent("LOOT_SLOT_CLEARED")
			context.mainWindow:UnregisterEvent("CHAT_MSG_SYSTEM")
			context.mainWindow:UnregisterEvent("ENCOUNTER_END")
		end
	else
		context.active = false
		context.lootmaster = false
		context.mainWindow:UnregisterEvent("LOOT_OPENED")
		context.mainWindow:UnregisterEvent("LOOT_CLOSED")
		context.mainWindow:UnregisterEvent("LOOT_SLOT_CLEARED")
		context.mainWindow:UnregisterEvent("CHAT_MSG_SYSTEM")
		context.mainWindow:UnregisterEvent("CHAT_MSG_ADDON")
		context.mainWindow:UnregisterEvent("ENCOUNTER_END")
		interface:showLoot()
	end
end

local function chatMsgAddon(identifier, message, sender)
	if identifier == ADDON_MESSAGE_IDENTIFIER then
		local messageData = decodeAddonMessage(message)
		local player = data:getCanonicalPlayerName(sender)
		local command = table.remove(messageData, 1)
		if context.lootmaster then
			if command == "PASS" then
				local lootSlot = tonumber(messageData[1])
				if context.lootSlot == lootSlot then
					local item = context.lootTable[context.lootSlot]
					local c, i
					for index, candidate in pairs(item.candidates) do
						if candidate.player == player then
							c = candidate
							i = index
							break
						end
					end
					if c then
						c.passed = true
						local receivedProg, receivedFarm = core:getNumReceivedItems(player)
						messenger:add(ADDON_MESSAGE_IDENTIFIER, encodeAddonMessage(
							"ROLL", 
							context.lootSlot, 
							i, 
							c.player,
							c.name,
							c.realm,
							nil,
							0,
							false,
							receivedProg,
							receivedFarm,
							c.isRaidMember
						), "RAID")
						interface.updateLootCandidates()
					end
				end
			end
		else
			if command == "LOOT_OPENED" then
				local lootSlot = tonumber(messageData[1])
				context.lootItemsTotal = tonumber(messageData[2])
				context.lootItemsReceived = (context.lootItemsReceived or 0) + 1
				for slot = 1,context.lootItemsTotal do
					context.lootTable[slot] = context.lootTable[slot] or {
						item = nil,
						itemText = "",
						texture = "Interface\\Icons\\INV_Misc_QuestionMark",
						quantity = 0,
						quality = 1,
						looted = false,
						eligibleCandidates = {},
						candidates = {}
					}
				end
				local item = context.lootTable[lootSlot]
				if messageData[3] then
					item.item = messageData[3]
				end
				if item.item then
					GetItemInfo(item.item)
					--local itemId = data:decodeItemLink(item.item).id
				end
				item.itemText = messageData[4]
				item.texture = messageData[5]
				item.quantity = tonumber(messageData[6])
				item.quality = tonumber(messageData[7])
				if context.lootItemsReceived == 1 then
					context.lootSlot = nil
				end
				if context.lootItemsReceived >= context.lootItemsTotal then
					context.encounterId = tonumber(messageData[8])
					context.isProgressBoss = messageData[9]
					interface:showLoot(context.encounterId, context.isProgressBoss)
				end
			elseif command == "CANDIDATE_SELECT" then
				local lootSlot = tonumber(messageData[1])
				local recipient = messageData[2]
				context.lootTable[lootSlot].recipient = recipient
				context.lootSlot = lootSlot
				interface:showLootItem(context.lootSlot)
			elseif command == "ENCOUNTER_SELECT" then
				context.encounterId = tonumber(messageData[1])
				context.isProgressBoss = messageData[2]
				interface:selectEncounter(context.encounterId, context.isProgressBoss)
			elseif command == "LOOT_CLOSED" then
				lootClosed()
			elseif command == "LOOT_SLOT_CLEARED" then
				lootSlotCleared(tonumber(messageData[1]))
			elseif command == "ITEM_SELECT" then
				context.lootSlot = tonumber(messageData[1])
				interface:showLootItem(context.lootSlot)
			elseif command == "ROLL" then
				local lootSlot = tonumber(messageData[1])
				local candidateIndex = tonumber(messageData[2])
				local player = messageData[3]
				local rollType = messageData[6]
				local roll = tonumber(messageData[7])
				local isMasterLooterRoll = messageData[8]
				context.receivedCache = context.receivedCache or {}
				if not context.receivedCache[player] then
					context.receivedCache[player] = {
						lastReceived = {}
					}
				end
				context.receivedCache[player].receivedProg = tonumber(messageData[9] or "0")
				context.receivedCache[player].receivedFarm = tonumber(messageData[10] or "0")
				local candidates = context.lootTable[lootSlot].candidates
				for index = 1,candidateIndex do
					if not candidates[index] then
						candidates[index] = {
							player = "",
							name = "",
							realm = ""
						}
					end
				end
				local candidate = candidates[candidateIndex]
				candidate.player = player
				candidate.name = messageData[4]
				candidate.realm = messageData[5]
				candidate.isRaidMember = messageData[11]
				if rollType then
					candidate.passed = false
					candidate[rollType] = roll
				else
					candidate.passed = true
				end
				if isMasterLooterRoll then
					if rollType == "greed" then
						candidate.need = nil
					end
					if rollType == "disenchant" then
						candidate.need = nil
						candidate.greed = nil
					end
				end
				interface.updateLootCandidates()
			end
		end
	end
end

local function addonLoaded(addonName)
	if addonName == "LootsturmLight" then
		data = LootsturmLightDataInterface:initialize()
		partyLootMethodChanged()
	end
end

local handleList = {
	["ADDON_LOADED"] = addonLoaded,
	["CHAT_MSG_ADDON"] = chatMsgAddon, 
	["CHAT_MSG_SYSTEM"] = chatMsgSystem, 
	-- ["CHAT_MSG_LOOT"] = processLootMessage, 
	-- ["CHAT_MSG_WHISPER"] = processWhisperMessage,
	["LOOT_OPENED"] = lootOpened,
	["LOOT_CLOSED"] = lootClosed,
	["ENCOUNTER_END"] = encounterEnd,
	["LOOT_SLOT_CLEARED"] = lootSlotCleared,
	["PARTY_LOOT_METHOD_CHANGED"] = partyLootMethodChanged,
	["RAID_ROSTER_UPDATE"] = partyLootMethodChanged,
	["GROUP_ROSTER_UPDATE"] = partyLootMethodChanged
}

local function onEvent(_, event, msg, msg2, _, sender)
	handleList[event](msg, msg2, sender)
end

local function slashcmdhandler(args, editbox)
	local argv = {}
	for v in string.gmatch(args, "[%S]+") do 
		table.insert(argv, v)
	end

	if #argv == 0 then
		print("LootsturmLight Heartbeat")
		return
	end
	
	if argv[1] == "typecache" then
		local successful = 0
		local failed = 0
		for index, playerName in pairs(data:getAllPlayers()) do
			for itemId, received in pairs(data:getPlayer(playerName).received) do
				if data:cacheItemType(itemId) then
					successful = successful + 1
				else
					failed = failed + 1
				end
			end
		end
		print(("Successfully added type of %d items, %d failed."):format(successful, failed))
	end
	
	if argv[1] == "sim" then
		if argv[2] == "on" then
			API = simulatorAPI
			API.loot = {}
			context.encounterId = 1896 -- The Desolate Host
			local simPlayer = data:getPlayer("Seleves-DerRatvonDalaran")
			local num = 0
			local maxNum = 7
			for id, items in pairs(simPlayer.received) do
				for index, item in pairs(items) do
					local name, link, _, _, _, _, _, _, _, texture, _ = GetItemInfo(item.link)
					if num < maxNum and name and link and texture then
						num = num + 1
						API.loot[num] = {
							itemLink = link,
							itemText = name,
							texture  = texture
						}
					end
				end
			end
			API.candidates = data:getAllPlayers()
			partyLootMethodChanged()
			lootOpened()
		elseif argv[2] == "off" then
			API = blizzardAPI
			lootClosed()
			partyLootMethodChanged()
		elseif argv[2] == "roll" then
			local who = tonumber(argv[3])
			if API.candidates[who] then
				chatMsgSystem(("%s würfelt. Ergebnis: %s (1-%s)"):format(API.candidates[who], argv[5], argv[4]))
			end
		end
	end
	
	if argv[1] == "handicap" then
		local numPlayers = 0
		local totalItems = 0
		for index, name in pairs(data:getAllPlayers()) do
			numPlayers = numPlayers + 1
			local player = data:getPlayer(name)
			local numItems = 0
			for id, itemGroup in pairs(player.received) do
				for i, item in pairs(itemGroup) do
					if item.source == SOURCE.PROGRESS then
						numItems = numItems + 1
					end
				end
			end
			totalItems = totalItems + numItems
			print(("%s: %d"):format(name, numItems))
		end
		print(("Total: %d, Avg: %.2f (Players: %d)"):format(totalItems, totalItems/numPlayers, numPlayers))
	end
	
	if argv[1] == "check" then
		local sources = {}
		for index, name in pairs(data:getAllPlayers()) do
			local player = data:getPlayer(name)
			for id, itemGroup in pairs(player.received) do
				for i, item in pairs(itemGroup) do
					sources[item.source] = (sources[item.source] or 0) + 1
				end
			end	
		end
		for source, count in pairs(sources) do
			print(("%s: %d"):format(source, count))
		end
	end
	
	if argv[1] == "clean" then
		print("this function is deactivated")
		-- print("start clean")
		-- local sources = {
			-- ["Grabmal HC"]                = { source = "farm"     , difficultyId = 15, encounterId = 0}, 
			-- ["Grabmal HC Sekundär"]       = { source = "second"   , difficultyId = 15, encounterId = 0}, 
			-- ["Grabmal NH"]                = { source = "other"    , difficultyId = 14, encounterId = 0}, 
			-- ["Grabmal NH Sekundär"]       = { source = "other"    , difficultyId = 14, encounterId = 0}, 
			-- ["Grabmal Mythisch"]          = { source = "progress" , difficultyId = 16, encounterId = 0}, 
			-- ["Grabmal Mythisch Sekundär"] = { source = "second"   , difficultyId = 16, encounterId = 0},
			-- ["progress"]                  = { source = "progress" }, 
			-- ["farm"]                      = { source = "farm"     }, 
			-- ["second"]                    = { source = "second"   }, 
			-- ["other"]                     = { source = "other"    }
		-- }
		-- local remaining = 0
		-- local deleted = 0
		-- for index, name in pairs(data:getAllPlayers()) do
			-- local player = data:getPlayer(name)
			-- local receivedNew = {}
			-- for id, itemGroup in pairs(player.received) do
				-- local receivedForIdNew = {}
				-- local empty = true
				-- for i, item in pairs(itemGroup) do
					-- if sources[item.source] then
						-- item.source       = sources[item.source].source
						-- item.encounterId  = 0
						-- if item.source == "progress" then
							-- item.difficultyId = 16
						-- elseif item.source == "farm" then
							-- item.difficultyId = 15
						-- elseif item.source == "second" then
							-- item.difficultyId = 15
						-- else
							-- item.difficultyId = 14
						-- end
						-- receivedForIdNew[#receivedForIdNew+1] = item
						-- empty = false
						-- remaining = remaining + 1
					-- else
						-- deleted = deleted + 1
					-- end
				-- end
				-- if not empty then
					-- receivedNew[id] = receivedForIdNew
				-- end
			-- end
			-- player.received = receivedNew
		-- end
		-- print(("finished clean, %d items deleted, %d items remaining"):format(deleted, remaining))
	end
end

core.rollNeed = function(self)
	RandomRoll(1, 100)
end

core.rollGreed = function(self)
	RandomRoll(1, 50)
end

core.rollDisenchant = function(self)
	RandomRoll(1, 10)
end

core.rollPass = function(self)
	messenger:add(ADDON_MESSAGE_IDENTIFIER, encodeAddonMessage("PASS", context.lootSlot), "RAID")
end

core.giveLoot = function(self)
	if context.lootmaster then
		local t = 0.1
		for slotId, loot in pairs(context.lootTable) do
			if not loot.looted and loot.recipient then
				for i = 1,40 do
					local name = API:GetMasterLootCandidate(slotId, i)
					if name and data:getCanonicalPlayerName(name) == loot.recipient then
						--print(("TESTMODUS: %s -> %s (%d)"):format(loot.item, loot.recipient, i))
						C_Timer.After(t, function() 
							--print(("TESTMODUS: %s -> %s (%d)"):format(loot.item, loot.recipient, i))
							GiveMasterLoot(slotId, i)
							--SendChatMessage(("%s -> %s (%d)"):format(loot.item, loot.recipient, i), "RAID")
						end)
						t= t + 0.1
						break
					end
				end
			end
		end
	end
end

core.setLootRecipient = function(self, recipient)
	if context.lootSlot then
		local item = context.lootTable[context.lootSlot]
		if item and not item.looted and (not recipient or item.eligibleCandidates[recipient]) then
			item.recipient = recipient
			if item.recipient then
				local c
				for index, candidate in pairs(item.candidates) do
					if candidate.player == recipient then
						c = candidate
						break
					end
				end
				if not c then
					local n, r = data:getPlayerNameAndRealm(recipient)
					local receivedProg, receivedFarm = self:getNumReceivedItems(recipient)
					c = {
						player = recipient,
						name = n,
						realm = r,
						disenchant = 0,
						isRaidMember = data:isRaidMember(recipient)
					}
					item.candidates[#item.candidates+1] = c
					i = #item.candidates
					messenger:add(ADDON_MESSAGE_IDENTIFIER, encodeAddonMessage(
						"ROLL", 
						context.lootSlot, 
						i, 
						recipient,
						n,
						r,
						"disenchant",
						0,
						true,
						receivedProg,
						receivedFarm,
						c.isRaidMember
					), "RAID")
				end
			end
			interface:showLootItem(context.lootSlot)
			messenger:add(ADDON_MESSAGE_IDENTIFIER, encodeAddonMessage("CANDIDATE_SELECT", context.lootSlot, item.recipient), "RAID")
		end
	end
end

core.selectEncounter = function(self, encounterId, isProgress)
	if encounterId then
		context.encounterId = encounterId
		context.isProgressBoss = data:isProgressBoss(context.encounterId, context.difficultyId)
		return context.isProgressBoss
	else
		context.isProgressBoss = isProgress
	end
	messenger:add(ADDON_MESSAGE_IDENTIFIER, encodeAddonMessage("ENCOUNTER_SELECT", context.encounterId, context.isProgressBoss), "RAID")
end

core.setNeedLevel = function(self, level)
	if context.lootSlot then
		local item = context.lootTable[context.lootSlot]
		if item and not item.looted and item.recipient then
			local c, i
			for index, candidate in pairs(item.candidates) do
				if candidate.player == item.recipient then
					c = candidate
					i = index
					break
				end
			end
			if c then
				local outLevel = ""
				local outRoll = 0
				if level == "pass" then
					c.passed = true
				elseif level == "need" then
					c.passed = false
					c.need = c.need or 0
				elseif level == "greed" then
					c.passed = false
					c.need = nil
					c.greed = c.greed or 0
				elseif level == "disenchant" then
					c.passed = false
					c.need = nil
					c.greed = nil
					c.disenchant = c.disenchant or 0
				end
				if level ~= "pass" then
					outLevel = level
					outRoll = c[level]
				end
				local n, r = data:getPlayerNameAndRealm(item.recipient)
				local receivedProg, receivedFarm = self:getNumReceivedItems(item.recipient)
				messenger:add(ADDON_MESSAGE_IDENTIFIER, encodeAddonMessage(
					"ROLL", 
					context.lootSlot, 
					i, 
					item.recipient,
					n,
					r,
					outLevel,
					outRoll,
					true,
					receivedProg,
					receivedFarm,
					c.isRaidMember
				), "RAID")
				interface:showLootItem(context.lootSlot)
			end
		end
	end
end

core.getNeedLevel = function(self)
	if context.lootSlot then
		local item = context.lootTable[context.lootSlot]
		if item and item.recipient then
			for index, candidate in pairs(item.candidates) do
				if candidate.player == item.recipient then
					if candidate.passed then
						return "pass"
					elseif candidate.need then
						return "need"
					elseif candidate.greed then
						return "greed"
					elseif candidate.disenchant then
						return "disenchant"
					else
						return nil
					end
				end
			end
		end
	end
	return nil
end

core.getLootRecipient = function(self, slotId)
	local item = context.lootTable[slotId or context.lootSlot]
	if item then
		return item.recipient
	else
		return nil
	end
end

core.hasMaximumRoll = function(self, player)
	if context.lootSlot then
		local playerRoll = -1
		local maxRoll = 0
		for index, candidate in pairs(context.lootTable[context.lootSlot].candidates) do
			local roll = (candidate.need or 0)*10000, (candidate.greed or 0)*100, (candidate.disenchant or 0)
			if not candidate.passed then
				maxRoll = math.max(maxRoll, roll)
				if candidate.player == player then
					playerRoll = roll
				end
			end
		end
		return playerRoll == maxRoll
	end
	return false
	
end

core.getNumLootCandidates = function(self)
	if context.lootSlot then
		return #context.lootTable[context.lootSlot].candidates
	else
		return 0
	end
end

core.getLootCandidate = function(self, index)
	if context.lootSlot then
		return context.lootTable[context.lootSlot].candidates[index]
	else
		return {}
	end
end

core.closeLoot = function(self)
	if context.lootmaster then
		CloseLoot()
	else
		context.mainWindow:Hide()
	end
end

core.getMasterLootCandidates = function(self, slotId)
	local candidates = {}
	for name, _ in pairs(context.lootTable[slotId].eligibleCandidates) do
		candidates[#candidates+1] = name
	end
	table.sort(candidates)
	return candidates
end

core.decodeItemLink = function(self, itemLink)
	return data:decodeItemLink(itemLink)
end

core.onMinimapButtonClick = function(self)
	if context.lootmaster or not context.active then
		interface:showItemHistory()
	elseif next(context.lootTable) then
		if context.mainWindow:IsShown() then
			context.mainWindow:Hide()
		else
			context.mainWindow:Show()
		end
	else
		interface:showItemHistory()
	end
end

core.lootCandidateClicked = function(self, index)
	if context.lootmaster then
		if context.lootSlot then
			local item = context.lootTable[context.lootSlot]
			if item and not item.looted then
				item.recipient = item.candidates[index].player
				interface:showLootItem(context.lootSlot)
				messenger:add(ADDON_MESSAGE_IDENTIFIER, encodeAddonMessage(
					"CANDIDATE_SELECT", 
					context.lootSlot, 
					item.recipient
				), "RAID")
			end
		end
	end
end

core.lootItemClicked = function(self, slotId)
	if context.lootmaster and context.lootTable[slotId] and not context.lootTable[slotId].looted and context.lootTable[slotId].item then
		context.lootSlot = slotId
		messenger:add(ADDON_MESSAGE_IDENTIFIER, encodeAddonMessage(
			"ITEM_SELECT", 
			context.lootSlot
		), "RAID")
		interface:showLootItem(context.lootSlot)
	end
end

core.getPlayerList = function(self)
	local playerList = {}
	for index, name in pairs(data:getAllPlayers()) do
		local n, r = data:getPlayerNameAndRealm(name)
		local hp, hf = data:getHandicap(name)
		playerList[#playerList+1] = {
			key               = name,
			name              = n,
			realm             = r,
			isRaidMember      = data:isRaidMember(name),
			handicap_progress = hp,
			handicap_farm     = hf
		}
	end
	return playerList
end

core.getLootItem = function(self, index)
	return context.lootTable[index]
end

core.getNumLootItems = function(self)
	return #context.lootTable
end

core.getCurrentLootSlot = function(self)
	return context.lootSlot
end

core.getItemList = function(self, playerName)
	local itemList = {}
	local sourceText = {
		[SOURCE.PROGRESS] = L.LOOT_SOURCE_PROGRESS,
		[SOURCE.FARM]     = L.LOOT_SOURCE_FARM,
		[SOURCE.SECOND]   = L.LOOT_SOURCE_SECONDARY,
		[SOURCE.OTHER]    = L.LOOT_SOURCE_OTHER
	}
	for id, items in pairs(data:getPlayer(playerName).received) do
		for index, item in pairs(items) do
			itemList[#itemList+1] = {
				id             = id,
				link           = item.link,
				source         = item.source,
				date           = item.date,
				sourceText     = sourceText[item.source] or item.source,
				encounterName  = EJ_GetEncounterInfo(item.encounterId) or L.UNKNOWN,
				difficultyName = GetDifficultyInfo(item.difficultyId) or L.UNKNOWN
			}
		end
	end
	return itemList
end

core.getBossList = function(self)
	local bossList = {}
	for encounterId, bossData in pairs(data:getBossData()) do
		bossList[#bossList+1] = {
			encounterId        = encounterId,
			encounterName      = EJ_GetEncounterInfo(encounterId),
			farmDifficulty     = bossData.farmDifficulty,
			farmDifficultyName = GetDifficultyInfo(bossData.farmDifficulty) or "-"
		}
	end
	return bossList
end

core.importEncounters = function(self)
	local allBosses = data:getBossData()
	local encounterId = 0
	local i = 1
	while true do
		local _, _, encounterId = EJ_GetEncounterInfoByIndex(i)
		if encounterId then
			if not allBosses[encounterId] then
				data:setBossData(encounterId)
			end
			i = i + 1
		else
			break
		end
	end
end

core.applyFarmDifficulty = function(self, farmDifficulty, bosses)
	for index, boss in pairs(bosses) do
		if boss.selected then 
			data:setBossData(boss.encounterId, farmDifficulty)
		end
	end
end

core.deleteSelectedBosses = function(self, bosses)
	local allBosses = data:getBossData()
	for index, boss in pairs(bosses) do
		if boss.selected then 
			allBosses[boss.encounterId] = nil
		end
	end
end

core.deleteSelectedItems = function(self, playerName, items)
	for index, item in pairs(items) do
		if item.selected then 
			data:deleteReceivedItem(playerName, item.link, item.source, item.date)
		end
	end
end

core.deletePlayer = function(self, playerName)
	data:deletePlayer(playerName)
end

core.setRaidMember = function(self, playerName, isRaidMember)
	data:setRaidMember(playerName, isRaidMember)
end

core.setHandicap = function(self, playerName, handicap_progress, handicap_farm)
	data:setHandicap(playerName, handicap_progress, handicap_farm)
end

core.isLootMaster = function(self)
	return context.lootmaster
end

core.getNumReceivedItems = function(self, playerName)
	if not context.receivedCache[playerName] then
		context.receivedCache[playerName] = {
			lastReceived = {}
		}
	end
	if not context.receivedCache[playerName].receivedProg or not context.receivedCache[playerName].receivedFarm then
		local buckets = data:getNumReceivedItems(playerName)
		local hp, hf = data:getHandicap(playerName)
		context.receivedCache[playerName].receivedProg = (buckets[SOURCE.PROGRESS] or 0) + (hp or 0)
		context.receivedCache[playerName].receivedFarm = (buckets[SOURCE.FARM] or 0) + (hf or 0)
	end
	return context.receivedCache[playerName].receivedProg, context.receivedCache[playerName].receivedFarm
end

core.getLastReceivedItem = function(self, playerName, lootslot)
	if not context.lootmaster then
		return nil
	end
	lootslot = lootslot or context.lootSlot
	if lootslot and context.lootTable[lootslot] and context.lootTable[lootslot].id then
		local itemType = data:cacheItemType(context.lootTable[lootslot].id)
		if itemType then
			if not context.receivedCache[playerName] then
				context.receivedCache[playerName] = {
					lastReceived = {}
				}
			end
			if not context.receivedCache[playerName].lastReceived[itemType] then
				local lastReceived = data:getLastReceivedItem(playerName, itemType, {[SOURCE.PROGRESS] = true, [SOURCE.FARM] = true})
				if lastReceived then
					context.receivedCache[playerName].lastReceived[itemType] = lastReceived
				else
					context.receivedCache[playerName].lastReceived[itemType] = "NA"
				end
			end
			local last = context.receivedCache[playerName].lastReceived[itemType]
			if last and last ~= "NA" then
				return last
			else
				return nil
			end
		end
	end
	return nil
end

core.getAverageReceivedItems = function(self)
	return data:getAverageReceivedItems()
end

context.mainWindow = interface:initialize(core)	
context.mainWindow:SetScript("OnEvent", onEvent)
context.mainWindow:RegisterEvent("ADDON_LOADED")
context.mainWindow:RegisterEvent("PARTY_LOOT_METHOD_CHANGED")
context.mainWindow:RegisterEvent("RAID_ROSTER_UPDATE")
context.mainWindow:RegisterEvent("GROUP_ROSTER_UPDATE")
RegisterAddonMessagePrefix(ADDON_MESSAGE_IDENTIFIER)

SlashCmdList["LOOTSTURMLIGHT"] = slashcmdhandler
