﻿LootsturmLightDataInterface = {}
local data = LootsturmLightDataInterface
local db

local invTypes = {
	["INVTYPE_2HWEAPON"]           = true,
	["INVTYPE_HOLDABLE"]           = true,
	["INVTYPE_RANGED"]             = true,
	["INVTYPE_RANGEDRIGHT"]        = true,
	["INVTYPE_SHIELD"]             = true,
	["INVTYPE_WEAPON"]             = true,
	["INVTYPE_WEAPONMAINHAND"]     = true,
	["INVTYPE_WEAPONMAINHAND_PET"] = true,
	["INVTYPE_WEAPONOFFHAND"]      = true,
	["INVTYPE_HEAD"]               = true,
	["INVTYPE_SHOULDER"]           = true,
	["INVTYPE_CLOAK"]              = true,
	["INVTYPE_CHEST"]              = true, -- same slot
	["INVTYPE_ROBE"]               = true,  --
	["INVTYPE_WRIST"]              = true,
	["INVTYPE_HAND"]               = true,
	["INVTYPE_WAIST"]              = true,
	["INVTYPE_LEGS"]               = true,
	["INVTYPE_FEET"]               = true,
	["INVTYPE_FINGER"]             = true,
	["INVTYPE_NECK"]               = true,
	["INVTYPE_TRINKET"]            = true
}
		
local relicTypes = {
	["Blood"]  = true,
	["Shadow"] = true,
	["Iron"]   = true,
	["Frost"]  = true,
	["Fire"]   = true,
	["Fel"]    = true,
	["Arcane"] = true,
	["Life"]   = true,
	["Wind"]   = true,
	["Holy"]   = true
}

data.initialize = function(self)
	LootsturmLightData                      = LootsturmLightData or {}
	LootsturmLightData.settings             = LootsturmLightData.settings or {}
	LootsturmLightData.players              = LootsturmLightData.players or {}
	LootsturmLightData.typeCache            = LootsturmLightData.typeCache or {}
	LootsturmLightData.bosses               = LootsturmLightData.bosses or {}
	db = LootsturmLightData
	return self
end

data.cacheItemType = function(self, itemId, forceCache)
	if not db.typeCache[itemId] or forceCache then
		local _, _, _, _, _, _, _, _, equipSlot = GetItemInfo(itemId)
		if equipSlot and invTypes[equipSlot] then
			if equipSlot == "INVTYPE_ROBE" then
				equipSlot = "INVTYPE_CHEST"
			end
			db.typeCache[itemId] = equipSlot
			return equipSlot
		else
			local _, _, relicType = C_ArtifactUI.GetRelicInfoByItemID(itemId)
			if relicType and relicTypes[relicType] then
				db.typeCache[itemId] = relicType
				return relicType
			else
				return nil
			end
		end
	else
		return db.typeCache[itemId]
	end
end

data.decodeItemLink = function(self, lItem)
	local subs = {}
	local dItem = {}
	for substring in lItem:gmatch('[^\|]+') do
		subs[#subs+1] = substring:sub(2)
	end
	dItem.color = subs[1]
	dItem.name = subs[3]:sub(2,-2)
	local tmp = {}
	for field in subs[2]:gmatch('[^:]+') do
		tmp[#tmp+1] = field
	end
	dItem.id     = tonumber(tmp[2])
	dItem.suffix = tonumber(tmp[8])
	dItem.bonuses = {}
	dItem.bonusCount = 0
	for i = 15, #tmp do
		local bid = tonumber(tmp[i])
		dItem.bonuses[bid] = i-14
		dItem.bonusCount = dItem.bonusCount + 1
	end
	dItem.equals = function(self, other)
		if self.id ~= other.id or self.suffix ~= other.suffix or self.bonusCount ~= other.bonusCount then
			return false
		end
		for bid, _ in pairs(self.bonuses) do
			if not other.bonuses[bid] then
				return false
			end
		end
		return true
	end
	return dItem
end

data.encodeItemLink = function(self, dItem)
	local bonusTable = {}
	for bonusId, position in pairs(dItem.bonuses) do
		bonusTable[position] = bonusId
	end
	local bonusString = ""
	for i = 1, dItem.bonusCount do
		bonusString = bonusString .. ":" .. tostring(bonusTable[i])
	end
	return string.format("|c%s|Hitem:%d:0:0:0:0:0:%d:0:0:0:0:0:%d%s|h[%s]|h|r", dItem.color, dItem.id, dItem.suffix, dItem.bonusCount, bonusString, dItem.name)
	--                                I E G G G G  S U L ? U D #B B$
	-- "|cffa335ee|Hitem:105562:0:0:0:0:0:0:1366426752:90:505:0:0|h[Handgelenksschützer der Zerstörung]|h|r"
end

data.getCanonicalPlayerName = function(self, playerName)
	local _, _, name, realm = playerName:find('([^%-]*)%-?(.*)')
	if realm == "" then
		realm = nil
	end
	realm = (realm or GetRealmName()):gsub(" ","")
	return name .. "-" .. realm
end

data.getPlayerNameAndRealm = function(self, playerName)
	local _, _, name, realm = playerName:find('([^%-]*)%-?(.*)')
	if realm == "" then
		realm = nil
	end
	realm = realm or GetRealmName()
	return name, realm
end

data.getAllPlayers = function(self)
	local t = {}
	for name, _ in pairs(db.players) do
		t[#t+1] = name
	end
	--table.sort(t)
	return t
end

data.getPlayer = function(self, playerName)
	return db.players[playerName]
end

data.addPlayer = function(self, playerName)
	playerName = self:getCanonicalPlayerName(playerName)
	if not db.players[playerName] then
		db.players[playerName] = {
			received = {}
		}
	end
end

data.deletePlayer = function(self, playerName)
	--playerName = self:getCanonicalPlayerName(playerName)
	db.players[playerName] = nil
end

data.setRaidMember = function(self, playerName, isRaidMember)
	if db.players[playerName] then
		db.players[playerName].isRaidMember = isRaidMember
	end
end

data.isRaidMember = function(self, playerName)
	return db.players[playerName] and db.players[playerName].isRaidMember
end

data.setHandicap = function(self, playerName, handicap_progress, handicap_farm)
	if db.players[playerName] then
		db.players[playerName].handicap_progress = handicap_progress or 0
		db.players[playerName].handicap_farm     = handicap_farm or 0
	end
end

data.getHandicap = function(self, playerName)
	if not db.players[playerName] then
		return nil, nil
	end
	return db.players[playerName].handicap_progress or 0, db.players[playerName].handicap_farm or 0
end

data.addReceivedItem = function(self, playerName, lItem, source, encounterId, difficultyId)
	playerName = self:getCanonicalPlayerName(playerName)
	local dItem = self:decodeItemLink(lItem)
	local receivedForId = db.players[playerName].received[dItem.id]
	if not receivedForId then
		receivedForId = {}
		db.players[playerName].received[dItem.id] = receivedForId
	end
	-- if not force then
		-- for _, item in pairs(receivedForId) do
			-- dItemExisting = self:decodeItemLink(item.link)
			-- if dItem:equals(dItemExisting) then
				-- return false
			-- end
		-- end
	-- end
	self:cacheItemType(dItem.id)
	local _, m, d, y = CalendarGetDate()
	receivedForId[#receivedForId+1] = {
		link         = lItem,
		source       = source,
		date         = y*1e4 + m*1e2 + d,
		encounterId  = encounterId,
		difficultyId = difficultyId
	}
	return true
end

data.deleteReceivedItem = function(self, playerName, lItem, source, recDate)
	playerName = self:getCanonicalPlayerName(playerName)
	local dItem = self:decodeItemLink(lItem)
	local receivedForId = db.players[playerName].received[dItem.id]
	if receivedForId then
		local receivedForIdNew = {}
		local notEmpty = false
		for _, item in pairs(receivedForId) do
			dItemExisting = self:decodeItemLink(item.link)
			if not dItem:equals(dItemExisting) or (source and source ~= item.source) or (recDate and recDate ~= item.date) then
				receivedForIdNew[#receivedForIdNew+1] = item
				notEmpty = true
			end
		end
		if notEmpty then
			db.players[playerName].received[dItem.id] = receivedForIdNew
		else
			db.players[playerName].received[dItem.id] = nil
		end
	end
end

data.getAverageReceivedItems = function(self, includeTrials)
	local buckets = {}
	local playerCount = 0
	for name, playerData in pairs(db.players) do
		if playerData.isRaidMember or includeTrials then
			playerCount = playerCount + 1
			for itemId, items in pairs(playerData.received) do
				for index, item in pairs(items) do
					buckets[item.source] = (buckets[item.source] or 0) + 1
				end
			end
		end
	end
	if playerCount > 0 then
		for bucket, count in pairs(buckets) do
			buckets[bucket] = count / playerCount
		end
	end
	return buckets
end

data.getNumReceivedItems = function(self, playerName) 
	playerName = self:getCanonicalPlayerName(playerName)
	local buckets = {}
	local playerCount = 0
	local playerData = db.players[playerName]
	if playerData then
		for itemId, items in pairs(playerData.received) do
			for index, item in pairs(items) do
				buckets[item.source] = (buckets[item.source] or 0) + 1
			end
		end
	end
	return buckets
end

data.getLastReceivedItem = function(self, playerName, itemType, sources)
	playerName = self:getCanonicalPlayerName(playerName)
	if db.players[playerName] then
		local returnItem = nil
		local maxDate = 0
		for itemId, items in pairs(db.players[playerName].received) do
			if db.typeCache[itemId] == itemType then
				for index, item in pairs(items) do
					if sources[item.source] and item.date > maxDate then
						returnItem = item.link
						maxDate = item.date
					end
				end
			end
		end
		return returnItem
	else
		return nil
	end
end

data.setBossData = function(self, encounterId, farmDifficulty)
	local boss = db.bosses[encounterId]
	if not db.bosses[encounterId] then
		db.bosses[encounterId] = {
			farmDifficulty = farmDifficulty or 0
		}
	else 
		db.bosses[encounterId].farmDifficulty = farmDifficulty or 0
	end
end

data.getBossData = function(self)
	return db.bosses
end

data.isProgressBoss = function(self, encounterId, difficultyId)
	if difficultyId < 14 or difficultyId > 16 then
		return false
	end
	return db.bosses[encounterId] and (db.bosses[encounterId].farmDifficulty < difficultyId)
end