﻿LootsturmLightInterface = {}
local interface = LootsturmLightInterface
local ui = LootsturmLightUI
local L = LootsturmLightLocalization

local ADDON_NAME = "Lootsturm Light"
local ADDON_VERSION = "1.5.2"

local EJ_GetEncounterInfo = EJ_GetEncounterInfo

local main
local core
local dataWindow

local needLevelToIndex = {
	["need"] = 1,
	["greed"] = 2,
	["disenchant"] = 3,
	["pass"] = 4
}

local indexToNeedLevel = {
	"need",
	"greed",
	"disenchant",
	"pass"
}

interface.initialize = function(self, coreObject)
	core = coreObject
	
	local tmp = EncounterJournal_LoadUI
	EncounterJournal_LoadUI = function(...)
		tmp(...)
		if EncounterJournalEncounterFrame then
			local copyEncountersButton = ui:CreateButton(L.BOSS_IMPORT, EncounterJournalEncounterFrame, 200, 20, 10, 10, function()
				core:importEncounters()
				interface.showBossList()
			end)
			copyEncountersButton:ClearAllPoints()
			copyEncountersButton:SetPoint("TOP", 0, -5)
		else
			print("LootsturmLight: Cannot create button in Encounter Journal")
		end
	end
	
	main = CreateFrame("Frame", "LootsturmLightMainWindow", UIParent, "BaseBasicFrameTemplate")
	main:SetPoint("CENTER", 0, 0)
	main:SetSize(640, 480)
	local t = main:CreateTexture(nil, "BACKGROUND")
	t:SetSize(640-4, 480-4)
	t:SetPoint("CENTER")
	t:SetColorTexture(0,0,0,0.5)
	main:SetFrameStrata("DIALOG")
	main:SetMovable(true)
	main:EnableMouse(true)
	main:RegisterForDrag("LeftButton")
	main:SetScript("OnDragStart", main.StartMoving)
	main:SetScript("OnDragStop", main.StopMovingOrSizing)
	main.CloseButton:SetScript("OnClick", function() core:closeLoot() end)
	main:Hide()
	
	main.titleFrame = CreateFrame("Frame", nil, main)
	main.titleFrame:SetSize(500,16)
	main.titleFrame:SetPoint("TOP", main, -10, -1)
	main.titleFrame.encounterList = ui:CreateSelectList(main.titleFrame, 0, 0, 300, {}, nil, function(list, index) 
		local _, encounterId = list:GetSelection()
		local isProgress = core:selectEncounter(encounterId)
		main.titleFrame.progressList:SetSelection(isProgress and 2 or 1, true)
	end)
	main.titleFrame.progressList = ui:CreateSelectList(main.titleFrame, 0, 310, 100, {
		{ caption = L.BOSS_STATUS_FARM     , value = false },
		{ caption = L.BOSS_STATUS_PROGRESS , value = true }
	}, 1, function(list, index) 
		local _, isProgress = list:GetSelection()
		core:selectEncounter(nil, isProgress)
	end)
	main.title = main:CreateFontString(nil, "ARTWORK", "GameFontWhite")
	main.title:SetJustifyH("CENTER")
	main.title:SetSize(500,16)
	main.title:SetPoint("TOP", main, -10, -5)

	-- local fonts = {
	-- "SystemFont_Outline_Small",
	-- "SystemFont_Outline",
	-- "SystemFont_InverseShadow_Small",
	-- "SystemFont_Med2",
	-- "SystemFont_Med3",
	-- "SystemFont_Shadow_Med3",
	-- "SystemFont_Huge1",
	-- "SystemFont_Huge1_Outline",
	-- "SystemFont_OutlineThick_Huge2",
	-- "SystemFont_OutlineThick_Huge4",
	-- "SystemFont_OutlineThick_WTF",
	-- "NumberFont_GameNormal",
	-- "NumberFont_Shadow_Small",
	-- "NumberFont_OutlineThick_Mono_Small",
	-- "NumberFont_Shadow_Med",
	-- "NumberFont_Normal_Med",
	-- "NumberFont_Outline_Med",
	-- "NumberFont_Outline_Large",
	-- "NumberFont_Outline_Huge",
	-- "Fancy22Font",
	-- "QuestFont_Huge",
	-- "QuestFont_Outline_Huge",
	-- "QuestFont_Super_Huge",
	-- "QuestFont_Super_Huge_Outline",
	-- "SplashHeaderFont",
	-- "Game11Font",
	-- "Game12Font",
	-- "Game13Font",
	-- "Game13FontShadow",
	-- "Game15Font",
	-- "Game18Font",
	-- "Game20Font",
	-- "Game24Font",
	-- "Game27Font",
	-- "Game30Font",
	-- "Game32Font",
	-- "Game36Font",
	-- "Game48Font",
	-- "Game48FontShadow",
	-- "Game60Font",
	-- "Game72Font",
	-- "Game11Font_o1",
	-- "Game12Font_o1",
	-- "Game13Font_o1",
	-- "Game15Font_o1",
	-- "QuestFont_Enormous",
	-- "DestinyFontLarge",
	-- "CoreAbilityFont",
	-- "DestinyFontHuge",
	-- "QuestFont_Shadow_Small",
	-- "MailFont_Large",
	-- "SpellFont_Small",
	-- "InvoiceFont_Med",
	-- "InvoiceFont_Small",
	-- "Tooltip_Med",
	-- "Tooltip_Small",
	-- "AchievementFont_Small",
	-- "ReputationDetailFont",
	-- "FriendsFont_Normal",
	-- "FriendsFont_Small",
	-- "FriendsFont_Large",
	-- "FriendsFont_UserText",
	-- "GameFont_Gigantic",
	-- "ChatBubbleFont",
	-- "Fancy16Font",
	-- "Fancy18Font",
	-- "Fancy24Font",
	-- "Fancy27Font",
	-- "Fancy30Font",
	-- "Fancy32Font",
	-- "Fancy48Font",
	-- "SystemFont_NamePlateFixed",
	-- "SystemFont_LargeNamePlateFixed",
	-- "SystemFont_NamePlate",
	-- "SystemFont_LargeNamePlate",
	-- "SystemFont_NamePlateCastBar",
	-- "GameFontNormal",
	-- "GameFontDisable",
	-- "GameFontHighlight",
	-- "GameFontNormalHuge",
	-- "GameFontHighlightHuge",
	-- "GameFontDisableHuge",
	-- "GameFontNormalSmall",
	-- "GameFontWhiteTiny",
	-- "GameFontNormalMed1",
	-- "GameFontNormalMed2",
	-- "GameFontNormalLarge",
	-- "GameFontNormalLargeOutline",
	-- "GameFontDisableSmall",
	-- "GameFontNormalHugeOutline2",
	-- "GameFontNormalOutline",
	-- "GameFontHighlightOutline",
	-- "QuestFontHighlight",
	-- "GameFontHighlightMedium",
	-- "GameFontBlackMedium",
	-- "GameFontBlackSmall",
	-- "GameFontRed",
	-- "GameFontRedLarge",
	-- "GameFontGreen",
	-- "GameFontBlack",
	-- "GameFontHighlightMed2",
	-- "GameFontHighlightLarge",
	-- "GameFontNormalMed3",
	-- "GameFontDisableLarge",
	-- "GameFontHighlightSmall",
	-- "GameFontHighlightSmallLeft",
	-- "GameFontDisableSmallLeft",
	-- "GameFontHighlightSmall2",
	-- "GameFontBlackSmall2",
	-- "GameFontNormalSmall2",
	-- "GameFontNormalLarge2",
	-- "GameFontNormalWTF2",
	-- "GameFontNormalWTF2Outline",
	-- "GameFontNormalHuge2",
	-- "GameFontNormalShadowHuge2",
	-- "GameFontNormalHugeOutline",
	-- "GameFontNormalHuge3",
	-- "GameFontNormalHuge3Outline",
	-- "GameTooltipHeaderText",
	-- "IMENormal",
	-- "IMEHighlight"
	-- }
	-- local i = -55
	-- for index, font in pairs(fonts) do
		-- ui:CreateText(font,main,516,15,(i-i%2)*15,-(i%2)*516,font)
		-- i = i+1
	-- end
	
	main.lootList = ui:CreateList(main, 70, 6, 6, function() 
		
		local lootFrame = CreateFrame("Frame", nil, nil, "TooltipBorderedFrameTemplate")
		lootFrame:SetSize(200, 64)
		
		lootFrame:SetScript("OnEnter",function(self)
			if self.item and self.item.item and self.item.item ~= "" then
				GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
				GameTooltip:SetHyperlink(self.item.item)
				GameTooltip:Show()
			end
		end)
		lootFrame:SetScript("OnLeave",function(self) GameTooltip:Hide() end)
		lootFrame:SetScript("OnMouseUp",function(self, button)
			core:lootItemClicked(self.index)
		end)
	
		local icon = lootFrame:CreateTexture(nil, "ARTWORK")
		icon:SetSize(48, 48)
		icon:SetPoint("TOPLEFT", 8, -8)
		lootFrame.icon = icon
		
		local itemText = lootFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlightOutline")
		itemText:SetJustifyH("LEFT")
		itemText:SetPoint("TOPLEFT", 64, -24)
		itemText:SetSize(128, 32)
		lootFrame.itemText = itemText
		
		local recipientText = lootFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlightOutline")
		recipientText:SetJustifyH("LEFT")
		recipientText:SetPoint("TOPLEFT", 64, -8)
		recipientText:SetSize(128, 16)
		lootFrame.recipientText = recipientText
		
		lootFrame.Update = function(self, index)
			self.index = index
			if index then
				local item = core:getLootItem(index)
				if core:getCurrentLootSlot() == index then
					self.Background:SetColorTexture(0.8, 0.8, 0.8, 0.8)
				else
					self.Background:SetColorTexture(0, 0, 0, 0.8)
				end
				if item.looted then
					self:SetAlpha(0.25)
				else
					self:SetAlpha(1)
				end
				self.item = item
				self.icon:SetTexture(item.texture or "Interface\\Icons\\INV_Misc_QuestionMark")
				local _, _, _, color = GetItemQualityColor(item.quality or 1)
				if item.quantity > 1 then
					self.itemText:SetText(("|c%s%s|r x %d"):format(color, item.itemText or "", item.quantity))
				else
					self.itemText:SetText(("|c%s%s|r"):format(color, item.itemText or ""))
				end
				self.recipientText:SetText(item.recipient or ("(%s)"):format(L.NOBODY))
			end
		end
		return lootFrame
	end)
	
	main.candidateList = ui:CreateList(main, 70, 222, {4, 3}, function() 
		local candidateFrame = CreateFrame("Frame", nil, nil, "TooltipBorderedFrameTemplate")
		candidateFrame:SetSize(130, 100)
		candidateFrame:SetScript("OnMouseUp",function(self, button) 
			core:lootCandidateClicked(self.index)
		end)
		
		candidateFrame:SetScript("OnEnter",function(self)
			if self.index then
				local lastReceived = core:getLastReceivedItem(self.candidate.player)
				GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
				if lastReceived then	
					GameTooltip:SetHyperlink(lastReceived)
					GameTooltip:AddLine(L.LAST_RECEIVED_TOOLTIP_ITEM:format(self.candidate.name), 1, 1, 1)
				else
					GameTooltip:AddLine(L.LAST_RECEIVED_TOOLTIP_NONE_CAPTION, 1, 1, 1)
					GameTooltip:AddLine(L.LAST_RECEIVED_TOOLTIP_NONE_TEXT)
				end
				GameTooltip:Show()
			end
		end)
		
		candidateFrame:SetScript("OnLeave",function(self) GameTooltip:Hide() end)
		
		candidateFrame.needIcon = candidateFrame:CreateTexture(nil, "ARTWORK")
		candidateFrame.needIcon:SetSize(32, 32)
		candidateFrame.needIcon:SetPoint("TOPLEFT", 16, -44)
		--candidateFrame.needIcon:SetTexture("Interface\\BUTTONS\\UI-GroupLoot-Dice-Up")
		--GameFontNormalHuge NumberFontNormalHuge
		candidateFrame.needText = ui:CreateText(nil, candidateFrame, 50, 32, 44, 48, "NumberFontNormalHuge", "CENTER")
		
		candidateFrame.nameText = ui:CreateText(nil, candidateFrame, 114, 20, 8, 8, "GameFontNormalLargeOutline", "CENTER")
		candidateFrame.realmText = ui:CreateText(nil, candidateFrame, 114, 10, 28, 8, "GameFontNormalSmall", "CENTER")
		
		candidateFrame.receivedText = ui:CreateText(nil, candidateFrame, 114, 10, 78, 8, "GameFontNormalSmall", "CENTER")
		
		candidateFrame.Update = function(self, index)
			self.index = index
			if index then
				local candidate = core:getLootCandidate(index)
				self.candidate = candidate
				if self.candidate.isRaidMember then
					self.nameText:SetText(candidate.name)
				else
					self.nameText:SetText(("[%s]"):format(candidate.name))
				end
				local isSelected = candidate.player == core.getLootRecipient()
				local color = isSelected and "ff000000" or "ffffffff"
				self.realmText:SetText(("|c%s%s|r"):format(color, candidate.realm))
				local needText = ""
				if candidate.passed then
					self.needIcon:SetTexture("Interface\\BUTTONS\\UI-GroupLoot-Pass-Up")
				elseif candidate.need then
					self.needIcon:SetTexture("Interface\\BUTTONS\\UI-GroupLoot-Dice-Up")
					needText = candidate.need > 0 and tostring(candidate.need) or "..."
				elseif candidate.greed then
					self.needIcon:SetTexture("Interface\\BUTTONS\\UI-GroupLoot-Coin-Up")
					needText = candidate.greed > 0 and tostring(candidate.greed) or "..."
				elseif candidate.disenchant then
					self.needIcon:SetTexture("Interface\\BUTTONS\\UI-GroupLoot-DE-Up")
					needText = candidate.disenchant > 0 and tostring(candidate.disenchant) or "..."
				else
					self.needIcon:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
				end
				
				if isSelected then
					self.Background:SetColorTexture(0.8, 0.8, 0.8, 0.8)
				else
					self.Background:SetColorTexture(0, 0, 0, 0.8)
				end
				
				if core:hasMaximumRoll(candidate.player) then
					self.needText:SetText(("|cffffd200%s|r"):format(needText))
				else
					self.needText:SetText(("|cffffffff%s|r"):format(needText))
				end
				
				local receivedProg, receivedFarm = core:getNumReceivedItems(candidate.player)
				self.receivedText:SetText(("|c%s%s: %d / %d|r"):format(color, L.RECEIVED, receivedProg, receivedFarm))
			end
		end
		
		return candidateFrame
	end)
	
	main.lootInfoText = main:CreateFontString(nil, "ARTWORK", "GameFontHighlightLarge")
	main.lootInfoText:SetJustifyH("CENTER")
	main.lootInfoText:SetSize(390, 80)
	main.lootInfoText:SetPoint("TOPLEFT", main, 230, -200)
	main.lootInfoText:SetText(L.WAIT_FOR_LOOT_MASTER)
	
	local lootPanel = CreateFrame("Frame", "LootsturmLightLootPanel", main, "TooltipBorderedFrameTemplate")
	lootPanel:SetSize(628, 40)
	lootPanel:SetPoint("TOPLEFT", 6, -26)
	lootPanel.giveButton = ui:CreateButton(L.GIVE_ALL, lootPanel, 115, 20, 10, 10, function() core:giveLoot() end)
	main.lootPanel = lootPanel
	
	local itemPanel = CreateFrame("Frame", "LootsturmLightItemPanel", lootPanel)
	itemPanel:SetSize(490, 36)
	itemPanel:SetPoint("RIGHT", -10, 0)
	
	itemPanel.needList = ui:CreateSelectList(itemPanel, 10, 15, 100, {
		{caption = "Bedarf", icon = "Interface\\BUTTONS\\UI-GroupLoot-Dice-Up"},
		{caption = "Sekundär", icon = "Interface\\BUTTONS\\UI-GroupLoot-Coin-Up"},
		{caption = "Sonstiges", icon = "Interface\\BUTTONS\\UI-GroupLoot-DE-Up"},
		{caption = "Passen", icon = "Interface\\BUTTONS\\UI-GroupLoot-Pass-Up"}
	}, 4, function(list, index) 
		core:setNeedLevel(indexToNeedLevel[index or 4])
	end)
	itemPanel.needList:ClearAllPoints()
	itemPanel.needList:SetPoint("RIGHT", -4, 0)
	
	itemPanel.candidateList = ui:CreateSelectList(itemPanel, 10, 15, 200, {}, nil, function(list, index) 
		local idx, recipient = list:GetSelection()
		if idx > 1 then
			core:setLootRecipient(recipient)
		else
			core:setLootRecipient(nil)
		end
	end)
	itemPanel.candidateList:ClearAllPoints()
	itemPanel.candidateList:SetPoint("RIGHT", itemPanel.needList, "LEFT", -10, 0)
	
	itemPanel.need = CreateFrame("Button", "LootsturmLightNeedButton", itemPanel)
	itemPanel.need:SetSize(32,32)
	itemPanel.need:SetPoint("LEFT", itemPanel, "LEFT", 10, -2)
	itemPanel.need:SetNormalTexture("Interface\\BUTTONS\\UI-GroupLoot-Dice-Up")
	itemPanel.need:SetPushedTexture("Interface\\BUTTONS\\UI-GroupLoot-Dice-Down")
	itemPanel.need:SetHighlightTexture("Interface\\BUTTONS\\UI-GroupLoot-Dice-Highlight")
	itemPanel.need:SetScript("OnEnter",function(self)
		GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
		GameTooltip:AddLine(L.NEED_LEVEL_NEED_TEXT)
		GameTooltip:AddLine(L.NEED_LEVEL_NEED_LINE_1, 1, 1, 1, true)
		GameTooltip:AddDoubleLine(L.ROLL_MACRO, "1-100")
		GameTooltip:Show()
	end)
	itemPanel.need:SetScript("OnLeave", function(self)
		GameTooltip:Hide()
	end)
	itemPanel.need:SetScript("OnClick",function(self) 
		core:rollNeed()
	end)
	itemPanel.greed = CreateFrame("Button", "LootsturmLightGreedButton", itemPanel)
	itemPanel.greed:SetSize(32,32)
	itemPanel.greed:SetPoint("LEFT", itemPanel.need, "RIGHT", 5, 0)
	itemPanel.greed:SetNormalTexture("Interface\\BUTTONS\\UI-GroupLoot-Coin-Up")
	itemPanel.greed:SetPushedTexture("Interface\\BUTTONS\\UI-GroupLoot-Coin-Down")
	itemPanel.greed:SetHighlightTexture("Interface\\BUTTONS\\UI-GroupLoot-Coin-Highlight")
	itemPanel.greed:SetScript("OnEnter",function(self)
		GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
		GameTooltip:AddLine(L.NEED_LEVEL_SECONDARY_TEXT)
		GameTooltip:AddLine(L.NEED_LEVEL_SECONDARY_LINE_1, 1,1,1, true)
		GameTooltip:AddLine(L.NEED_LEVEL_SECONDARY_LINE_2, 1,1,1, true)
		GameTooltip:AddTexture("Interface\\CHATFRAME\\ChatFrameExpandArrow")
		GameTooltip:AddLine(L.NEED_LEVEL_SECONDARY_LINE_3, 1,1,1, true)
		GameTooltip:AddTexture("Interface\\CHATFRAME\\ChatFrameExpandArrow")
		GameTooltip:AddDoubleLine(L.ROLL_MACRO, "1-50")
		GameTooltip:Show()
	end)
	itemPanel.greed:SetScript("OnLeave", function(self)
		GameTooltip:Hide()
	end)
	itemPanel.greed:SetScript("OnClick",function(self) 
		core:rollGreed()
	end)
	itemPanel.disenchant = CreateFrame("Button", "LootsturmLightDisenchantButton", itemPanel)
	itemPanel.disenchant:SetSize(32,32)
	itemPanel.disenchant:SetPoint("LEFT", itemPanel.greed, "RIGHT", 5, 0)
	itemPanel.disenchant:SetNormalTexture("Interface\\BUTTONS\\UI-GroupLoot-DE-Up")
	itemPanel.disenchant:SetPushedTexture("Interface\\BUTTONS\\UI-GroupLoot-DE-Down")
	itemPanel.disenchant:SetHighlightTexture("Interface\\BUTTONS\\UI-GroupLoot-DE-Highlight")
	itemPanel.disenchant:SetScript("OnEnter",function(self)
		GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
		GameTooltip:AddLine(L.NEED_LEVEL_OTHER_TEXT)
		GameTooltip:AddLine(L.NEED_LEVEL_OTHER_LINE_1, 1,1,1, true)
		GameTooltip:AddLine(L.NEED_LEVEL_OTHER_LINE_2, 1,1,1, true)
		GameTooltip:AddTexture("Interface\\CHATFRAME\\ChatFrameExpandArrow")
		GameTooltip:AddLine(L.NEED_LEVEL_OTHER_LINE_3, 1,1,1, true)
		GameTooltip:AddTexture("Interface\\CHATFRAME\\ChatFrameExpandArrow")
		GameTooltip:AddLine(L.NEED_LEVEL_OTHER_LINE_4, 1,1,1, true)
		GameTooltip:AddTexture("Interface\\CHATFRAME\\ChatFrameExpandArrow")
		GameTooltip:AddLine(L.NEED_LEVEL_OTHER_LINE_5, 1,1,1, true)
		GameTooltip:AddTexture("Interface\\CHATFRAME\\ChatFrameExpandArrow")
		GameTooltip:AddDoubleLine(L.ROLL_MACRO, "1-10")
		GameTooltip:Show()
	end)
	itemPanel.disenchant:SetScript("OnLeave", function(self)
		GameTooltip:Hide()
	end)
	itemPanel.disenchant:SetScript("OnClick",function(self) 
		core:rollDisenchant()
	end)
	itemPanel.pass = CreateFrame("Button", "LootsturmLightDisenchantButton", itemPanel)
	itemPanel.pass:SetSize(32,32)
	itemPanel.pass:SetPoint("LEFT", itemPanel.disenchant, "RIGHT", 5, 2)
	itemPanel.pass:SetNormalTexture("Interface\\BUTTONS\\UI-GroupLoot-Pass-Up")
	itemPanel.pass:SetPushedTexture("Interface\\BUTTONS\\UI-GroupLoot-Pass-Down")
	itemPanel.pass:SetHighlightTexture("Interface\\BUTTONS\\UI-GROUPLOOT-PASS-HIGHLIGHT")
	itemPanel.pass:SetScript("OnEnter",function(self)
		GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
		GameTooltip:AddLine(L.NEED_LEVEL_PASS)
		GameTooltip:AddLine(L.NEED_LEVEL_PASS_LINE_1, 1,1,1, true)
		GameTooltip:Show()
	end)
	itemPanel.pass:SetScript("OnLeave", function(self)
		GameTooltip:Hide()
	end)
	itemPanel.pass:SetScript("OnClick",function(self) 
		core:rollPass()
	end)
	main.itemPanel = itemPanel
	
	self:createMinimapButton(core.onMinimapButtonClick)
	
	dataWindow = CreateFrame("Frame", "LootsturmLightDataWindow", UIParent, "BaseBasicFrameTemplate")
	dataWindow:SetPoint("CENTER",0,0)
	dataWindow:SetSize(526,444)
	local t = dataWindow:CreateTexture(nil, "BACKGROUND")
	t:SetSize(522,440)
	t:SetPoint("CENTER")
	t:SetColorTexture(0,0,0,0.5)
	dataWindow:SetFrameStrata("DIALOG")
	dataWindow:SetFrameLevel(20)
	dataWindow:SetMovable(true)
	dataWindow:EnableMouse(true)
	dataWindow:RegisterForDrag("LeftButton")
	dataWindow:SetScript("OnDragStart", dataWindow.StartMoving)
	dataWindow:SetScript("OnDragStop", function(f) 
		f:StopMovingOrSizing()
	end)
	dataWindow.title = dataWindow:CreateFontString(nil, "ARTWORK", "GameFontWhite")
	dataWindow.title:SetJustifyH("CENTER")
	dataWindow.title:SetSize(500,16)
	dataWindow.title:SetPoint("TOP", dataWindow, -10, -5)
	dataWindow:Hide()
	
	PanelTemplates_SetNumTabs(dataWindow, 2)
	local playersTab = CreateFrame("Button", "$parentTab1", dataWindow, "TabButtonTemplate")
	playersTab:SetPoint("BOTTOMLEFT",dataWindow,"TOPLEFT", 5, 0)
	playersTab:SetText(L.RAID_TAB)
	PanelTemplates_TabResize(playersTab)
	playersTab:SetScript("OnClick", function(self, button)
		interface:showItemHistory()
	end)
	local bossesTab = CreateFrame("Button", "$parentTab2", dataWindow, "TabButtonTemplate")
	bossesTab:SetPoint("LEFT",playersTab,"RIGHT", 0, 0)
	bossesTab:SetText(L.BOSSES_TAB)
	PanelTemplates_TabResize(bossesTab)
	bossesTab:SetScript("OnClick", function(self, button) 
		interface:showBossList()
	end)
	
	-- player list frame
	local f = CreateFrame("Frame", nil, dataWindow)
	f:SetSize(516,354)
	f:SetPoint("TOPLEFT", 4,-30)
	f.playerList = ui:CreateTable(f, 0, 0, 375, 516, {}, 
	{
		{ name = "name",   type = "link",   width = 100, caption = L.PLAYER_LIST_COL_NAME,   align = "LEFT", onMouseUp = function(frame, index)
			interface:showItemHistory(dataWindow.playerListFrame.playerList:GetDataSource()[index].key)
		end },
		{ name = "realm",   type = "text",   width = 150, caption = L.PLAYER_LIST_COL_REALM,   align = "LEFT" },
		{ name = "isRaidMember", type = "bool", width = 50, caption = L.PLAYER_LIST_COL_PERMANENT,  onMouseUp = function(frame, index) 
			core:setRaidMember(dataWindow.playerListFrame.playerList:GetDataSource()[index].key, not dataWindow.playerListFrame.playerList:GetDataSource()[index].isRaidMember)
			dataWindow.playerListFrame:Update()
		end }, 
		{ name = "handicap_progress",   type = "action",   width =  75, caption = L.PLAYER_LIST_COL_HANDICAP_PROGRESS,   align = "LEFT", onMouseUp = function(tableObj, index) 
			local ds = tableObj:GetDataSource()[index]
			ui:InputPrompt(L.PLAYER_LIST_SET_HANDICAP_PROGRESS:format(ds.name, ds.realm), ds.handicap_progress, function(obj, newValue) 
				core:setHandicap(ds.key, tonumber(newValue), ds.handicap_farm)
				dataWindow.playerListFrame:Update()
			end)
		end	},
		{ name = "handicap_farm",   type = "action",   width =  75, caption = L.PLAYER_LIST_COL_HANDICAP_FARM,   align = "LEFT", onMouseUp = function(tableObj, index) 
			local ds = tableObj:GetDataSource()[index]
			ui:InputPrompt(L.PLAYER_LIST_SET_HANDICAP_FARM:format(ds.name, ds.realm), ds.handicap_farm, function(obj, newValue) 
				core:setHandicap(ds.key, ds.handicap_progress, tonumber(newValue))
				dataWindow.playerListFrame:Update()
			end)
		end },
		{ name = "delete", type = "action", width =  50, caption = L.PLAYER_LIST_COL_DELETE, align = "LEFT", onMouseUp = function(frame, index) 
			ui:ConfirmDialog(
				L.PLAYER_LIST_CONFIRM_DELETE:format(dataWindow.playerListFrame.playerList:GetDataSource()[index].name, dataWindow.playerListFrame.playerList:GetDataSource()[index].realm), 
				function()
					core:deletePlayer(dataWindow.playerListFrame.playerList:GetDataSource()[index].key)
					dataWindow.playerListFrame:Update()
				end
			)
		end }
	})
	f.playerList:Sort(1)
	f.avgText = ui:CreateText("", f, 516, 16, 375, 0, nil, "CENTER")
	f.Update = function(self)
		--getAverageReceivedItems
		local avgReceived = core:getAverageReceivedItems()
		self.avgText:SetText(L.PLAYER_LIST_AVERAGE_RECEIVED:format(avgReceived.progress or 0, avgReceived.farm or 0))
		self.playerList:Update(core:getPlayerList())
	end
	dataWindow.playerListFrame = f
	
	-- item list frame
	f = CreateFrame("Frame", nil, dataWindow)
	f:SetSize(516,354)
	f:SetPoint("TOPLEFT", 4,-30)
	f.itemList = ui:CreateTable(f, 0, 0, 380, 516, {}, 
	{
		{ name = "link",           type = "text",   width = 200, caption = L.ITEM_LIST_COL_ITEM, align = "LEFT",
			onEnter = function(frame, index) 
				local item = dataWindow.itemListFrame.itemList:GetDataSource()[index]
				local _, ilink = GetItemInfo(item.link)
				GameTooltip:SetOwner(frame, "ANCHOR_CURSOR") 
				GameTooltip:SetHyperlink(ilink or item.link) 
				GameTooltip:Show()
			end, onLeave = function(frame, index) GameTooltip:Hide() end
		},
		{ name = "sourceText",     type = "text",   width =  60, caption = L.ITEM_LIST_COL_NEED,     align = "LEFT" },
		{ name = "encounterName",  type = "text",   width = 110, caption = L.ITEM_LIST_COL_BOSS,       align = "LEFT" },
		{ name = "difficultyName", type = "text",   width =  60, caption = L.ITEM_LIST_COL_DIFFICULTY,      align = "LEFT" },
		{ name = "date",           type = "text",   width =  70, caption = L.ITEM_LIST_COL_DATE,   align = "LEFT" }
	}, "multiple")
	f.itemList:Sort(-5)
	f.backButton = ui:CreateButton(L.ITEM_LIST_BACK,f,150,20,380,10, function() interface:showItemHistory() end)
	f.deleteButton = ui:CreateButton(L.ITEM_LIST_DELETE_SELECTION,f,150,20,380,200, function() 
		core:deleteSelectedItems(dataWindow.itemListFrame.playerName, dataWindow.itemListFrame.itemList:GetDataSource())
		dataWindow.itemListFrame.itemList:Update(core:getItemList(dataWindow.itemListFrame.playerName))
	end)
	dataWindow.itemListFrame = f
	
	-- bosses frame
	f = CreateFrame("Frame", nil, dataWindow)
	f:SetSize(516,354)
	f:SetPoint("TOPLEFT", 4,-30)
	f.bossList = ui:CreateTable(f, 0, 0, 380, 500, {}, 
	{	
		{ name = "encounterId",        type = "text", width =  60, caption = L.BOSS_LIST_COL_ID,         align = "LEFT"},
		{ name = "encounterName",      type = "text", width = 274, caption = L.BOSS_LIST_COL_NAME,       align = "LEFT"},
		{ name = "farmDifficultyName", type = "text", width = 150, caption = L.BOSS_LIST_COL_FARM, align = "LEFT", sortBy = "farmDifficulty"}
	}, "multiple")
	f.bossList:Sort(-3)
	f.farmStatusList = ui:CreateSelectList(f, 380, 10, 100, {
		{ caption = "-"                  , value =  0 },
		{ caption = GetDifficultyInfo(14), value = 14 },
		{ caption = GetDifficultyInfo(15), value = 15 },
		{ caption = GetDifficultyInfo(16), value = 16 }
	}, 1)
	f.applyButton = ui:CreateButton(L.BOSS_LIST_APPLY,f,170,20,380,120, function()
		local _, status = dataWindow.bossListFrame.farmStatusList:GetSelection()
		core:applyFarmDifficulty(status, dataWindow.bossListFrame.bossList:GetDataSource())
		dataWindow.bossListFrame.bossList:Update(core:getBossList())
	end)
	f.deleteButton = ui:CreateButton(L.BOSS_LIST_DELETE,f,150,20,380,300, function()
		core:deleteSelectedBosses(dataWindow.bossListFrame.bossList:GetDataSource())
		dataWindow.bossListFrame.bossList:Update(core:getBossList())
	end)
	
	dataWindow.bossListFrame = f
	
	return main
end

interface.showLoot = function(self, encounterId, isProgress, encounterList)
	local numItems = core:getNumLootItems()
	main.lootList:Update(numItems)
	if core:isLootMaster() then
		main.titleFrame.progressList:SetSelection(isProgress and 2 or 1, true)
		if encounterList then
			local selectedEncounter
			for i = 1,#encounterList do
				if encounterList[i].value == encounterId then
					selectedEncounter = i
					break
				end
			end
			main.titleFrame.encounterList:SetItems(encounterList, selectedEncounter or 1)
		end
		main.titleFrame:Show()
		main.title:Hide()
		main.lootPanel.giveButton:Show()
		main.itemPanel.candidateList:Show()
		main.itemPanel.needList:Show()
	else
		main.titleFrame:Hide()
		if encounterId then
			self:selectEncounter(encounterId, isProgress)
		else
			main.title:SetText(ADDON_NAME .. " " .. ADDON_VERSION)
		end
		main.title:Show()
		main.lootPanel.giveButton:Hide()
		main.itemPanel.candidateList:Hide()
		main.itemPanel.needList:Hide()
	end
	if numItems > 0 then
		main.lootInfoText:Show()
		main.itemPanel:Hide()
		self:updateLootCandidates()
		main:Show()
	else
		main:Hide()
	end
end

interface.showLootItem = function(self, index)
	if index then
		main.lootList:ScrollTo(index)
		local recipient = core:getLootRecipient(index)
		if core:isLootMaster() then
			local candidateList = {{caption = ("(%s)"):format(L.NOBODY)}}
			local selectedIndex = 1
			for idx, candidate in pairs(core:getMasterLootCandidates(index)) do
				candidateList[#candidateList+1] = {caption = candidate}
				if candidate == recipient then
					selectedIndex = idx+1
				end
			end
			main.itemPanel.candidateList:SetItems(candidateList, selectedIndex)
			main.itemPanel.needList:SetSelection(needLevelToIndex[core:getNeedLevel() or "pass"], true)
		end
		main.candidateList:Show()
		self:updateLootCandidates()
		main.itemPanel:Show()
		main.lootInfoText:Hide()
	else
		main.lootInfoText:Show()
		main.lootList:Update()
		main.candidateList:Hide()
		main.itemPanel:Hide()
	end
end

interface.selectEncounter = function(self, encounterId, isProgress)
	if not core:isLootMaster() then
		main.title:SetText(("%s %s - %s (%s)"):format(ADDON_NAME, ADDON_VERSION, EJ_GetEncounterInfo(encounterId) or L.BOSS_UNKNOWN, isProgress and L.BOSS_STATUS_PROGRESS or L.BOSS_STATUS_FARM))
	end
end

interface.updateLootCandidates = function(self)
	main.candidateList:Update(core:getNumLootCandidates())
end

interface.setLooted = function(self, index)
	main.lootList:Update()
end

interface.showItemHistory = function(self, playerName)
	PanelTemplates_SetTab(dataWindow, 1)
	dataWindow.bossListFrame:Hide()
	if playerName then
		dataWindow.title:SetText(L.TITLE_RECEIVED_ITEMS_PLAYER:format(playerName))
		dataWindow.itemListFrame:Show()
		dataWindow.itemListFrame.playerName = playerName
		dataWindow.itemListFrame.itemList:Update(core:getItemList(dataWindow.itemListFrame.playerName))
		dataWindow.playerListFrame:Hide()
	else
		dataWindow.title:SetText(L.TITLE_RECEIVED_ITEMS)
		dataWindow.playerListFrame:Update()
		dataWindow.itemListFrame:Hide()
		dataWindow.playerListFrame:Show()
	end
	dataWindow:Show()
end

interface.showBossList = function(self)
	PanelTemplates_SetTab(dataWindow, 2)
	dataWindow.title:SetText("Bosse")
	dataWindow.playerListFrame:Hide()
	dataWindow.itemListFrame:Hide()
	dataWindow.bossListFrame.bossList:Update(core:getBossList())
	dataWindow.bossListFrame:Show()
	dataWindow:Show()
end

interface.createMinimapButton = function(self, onMinimapButtonClick)
	local minimapButton = CreateFrame("Button", "LootsturmLightMinimapButton", Minimap)
	minimapButton:SetSize(20,20)
	minimapButton:SetFrameStrata("MEDIUM")
	minimapButton:SetPoint("CENTER", -49.5, -49.5)
	minimapButton:SetMovable(true)
	minimapButton:SetUserPlaced(true)
	minimapButton:SetNormalTexture("Interface\\GROUPFRAME\\UI-Group-MasterLooter")
	minimapButton:SetPushedTexture("Interface\\GROUPFRAME\\UI-Group-MasterLooter")
	minimapButton:SetHighlightTexture("Interface\\GROUPFRAME\\UI-Group-MasterLooter")

	minimapButton:SetScript("OnMouseDown", function(self, button)
		if IsShiftKeyDown() or button == "RightButton" then
			self:SetScript("OnUpdate", function(self)
				local centerX, centerY = Minimap:GetCenter()
				local x, y = GetCursorPosition()
				x, y = x / self:GetEffectiveScale() - centerX, y / self:GetEffectiveScale() - centerY
				local vecLen = 80 / math.sqrt(x^2 + y^2) -- Minimap radius = 80
				centerX, centerY = x * vecLen, y * vecLen
				self:ClearAllPoints()
				self:SetPoint("CENTER", centerX, centerY)
			end)
		end
	end)
	minimapButton:SetScript("OnMouseUp", function(self)
		self:SetScript("OnUpdate", nil)
	end)
	
	minimapButton.clickHandler = onMinimapButtonClick
	minimapButton:SetScript("OnClick", function(self, button)
		if IsShiftKeyDown() or button == "RightButton" then return end
		self:clickHandler()
	end)
	minimapButton:SetScript("OnEnter", function(self)
		GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
		GameTooltip:SetText(ADDON_NAME, 1, 1, 1)
		GameTooltip:Show()
	end)
	minimapButton:SetScript("OnLeave", function(self)
		GameTooltip:Hide()
	end)
	self.minimapButton = minimapButton
end
