﻿LootsturmLightUI = {}
local ui = LootsturmLightUI
local IsControlKeyDown, IsShiftKeyDown, IsModifierKeyDown = IsControlKeyDown, IsShiftKeyDown, IsModifierKeyDown
ui.widgetNum = 0

ui.GetWidgetName = function(self)
	self.widgetNum = self.widgetNum + 1
	return "LootsturmLightWidget"..self.widgetNum
end

ui.AddBorder = function(self, frame, borderWidth)		
	local width, height = frame:GetSize()
	borderWidth = borderWidth or 2
	local t
	t = frame:CreateTexture(nil,"BORDER")
	t:SetSize(width,borderWidth)
	t:SetPoint("TOP")
	t:SetColorTexture(1, 1, 1, 0.5)
	t:SetGradient("HORIZONTAL", 1, 1, 1, 0.5, 0.5, 0.5)
	
	t = frame:CreateTexture(nil,"BORDER")
	t:SetSize(borderWidth,height-2*borderWidth)
	t:SetPoint("LEFT")
	t:SetColorTexture(1, 1, 1, 0.5)
	t:SetGradient("VERTICAL", 0.5, 0.5, 0.5, 1, 1, 1)
	
	t = frame:CreateTexture(nil,"BORDER")
	t:SetSize(width,borderWidth)
	t:SetPoint("BOTTOM")
	t:SetColorTexture(1, 1, 1, 0.5)
	t:SetGradient("HORIZONTAL", 0.5, 0.5, 0.5, 0, 0, 0)
	
	t = frame:CreateTexture(nil,"BORDER")
	t:SetSize(borderWidth,height-2*borderWidth)
	t:SetPoint("RIGHT")
	t:SetColorTexture(1, 1, 1, 0.5)
	t:SetGradient("VERTICAL", 0, 0, 0, 0.5, 0.5, 0.5)
end

ui.InputPrompt = function(self, caption, pretext, handle)
	StaticPopupDialogs["LSL_TextInput"] = {
		text = caption,
		button1 = ACCEPT,
		button2 = CANCEL,
		hasEditBox = 1,
		maxLetters = 150,
		OnAccept = function(self) 
			self:data(self.editBox:GetText())
			end,
		EditBoxOnEnterPressed = function(self)
			self:GetParent():data(self.editBox:GetText())
			self:GetParent():Hide()
		end,
		OnShow = function(self)
			self.editBox:SetText(pretext or "")
			self.editBox:HighlightText()
			self.editBox:SetFocus()
		end,
		OnHide = function(self)
			ChatEdit_FocusActiveWindow()
			self.editBox:SetText("")
		end,
		timeout = 0,
		exclusive = 1,
		whileDead = 1,
		hideOnEscape = 1
	};
	StaticPopup_Show("LSL_TextInput", "", "", handle); 
end

ui.ConfirmDialog = function(self, caption, handle)
	StaticPopupDialogs["LSL_ConfirmDialog"] = {
		text = caption,
		button1 = ACCEPT,
		button2 = CANCEL,
		hasEditBox = false,
		OnAccept = function(self) 
			self:data()
			end,
		timeout = 0,
		exclusive = 1,
		whileDead = 1,
		hideOnEscape = 1
	};
	StaticPopup_Show("LSL_ConfirmDialog", "", "", handle); 
end

ui.CreateButton = function(self, caption, parent, width, height, top, left, onClick)
	local button = CreateFrame("Button",self:GetWidgetName(),parent,"UIPanelButtonTemplate");
	button:SetPoint("TOPLEFT",left,-top)
	button:SetSize(width,height)
	button:SetText(caption)
	if onClick then
		button:SetScript("OnClick", onClick)
	end
	return button
end

ui.CreateSelectList = function(self, parent, top, left, width, items, selection, onChange)
	local f = CreateFrame("Frame", self:GetWidgetName(), parent, "InputBoxTemplate")
	f:SetPoint("TOPLEFT",left, -top)
	f:SetSize(width, 20)
	f.button = CreateFrame("Button",nil,f);
	f.button:SetPoint("RIGHT",2,0)
	f.button:SetSize(24,24)
	f.button:SetNormalTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Up")
	f.button:SetPushedTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Down")
	f.button:SetScript("OnClick", function(self)
		if self:GetParent().menu:IsShown() then
			self:GetParent().menu:Hide()
		else
			self:GetParent().menu:Show()
		end
	end)
	f:SetScript("OnMouseUp", function(self)
		if self.menu:IsShown() then
			self.menu:Hide()
		else
			self.menu:Show()
		end
	end)
	f.icon = f:CreateTexture(nil, "ARTWORK")
	f.icon:SetSize(14,14)
	f.icon:SetPoint("LEFT",0,0)	
	f.text = f:CreateFontString(nil, "ARTWORK", "GameTooltipText")
	f.text:SetPoint("LEFT",20,0)
	f.text:SetSize(width-40,16)
	f.onChange = onChange
	f.itemCount = 0
	f.menuItems = {}
	f.createMenuItem = function(self, id)
		local m = CreateFrame("Frame", nil, self.menu)
		m.id = id
		m:SetSize(self:GetWidth(), 18)
		m:SetPoint("TOP",5,14-id*18)
		m.icon = m:CreateTexture(nil, "ARTWORK")
		m.icon:SetPoint("LEFT",0,0)
		m.icon:SetSize(16,16)
		m.text = m:CreateFontString(nil, "ARTWORK", "GameTooltipText")
		m.text:SetPoint("LEFT",20,0)
		m.text:SetSize(self:GetWidth()-40,16)
		m.texture = m:CreateTexture(nil, "BACKGROUND")
		m.texture:SetPoint("LEFT",18,0)
		m.texture:SetSize(self:GetWidth()-30,16)
		m:SetScript("OnEnter", function(self) self.texture:SetTexture("Interface\\Buttons\\UI-ListBox-Highlight") end)
		m:SetScript("OnLeave", function(self) self.texture:SetTexture(nil) end)
		m:SetScript("OnMouseUp", function(self) self:GetParent():GetParent():SetSelection(self.id) end)
		return m
	end
	f.GetSelection = function(self) 
		if self.selection then
			return self.selection, self.menuItems[self.selection].item.value or self.menuItems[self.selection].item.caption
		else
			return nil, nil
		end
	end
	f.SetSelection = function(self, id, silent)
		local oldSelection = self.selection
		if id and self.menuItems[id] then
			self.icon:SetTexture(self.menuItems[id].icon:GetTexture())
			self.text:SetText(self.menuItems[id].text:GetText())
			self.selection = id
		else
			self.icon:SetTexture()
			self.text:SetText()
			self.selection = nil
		end
		self.menu:Hide()
		if not silent and self.onChange then --and oldSelection ~= self.selection 
			self.onChange(self, self.selection)
		end
	end
	f.SetItems = function(self, items, selection)
		items = items or {}
		if #items > #self.menuItems then
			for i = #self.menuItems+1,#items do
				self.menuItems[#self.menuItems+1] = self:createMenuItem(i)
			end
		end
		self.itemCount = #items
		self.menu:SetSize(self:GetWidth(), 18*#items+10)
		for i=1,#self.menuItems do
			if i <= self.itemCount then
				self.menuItems[i].item = items[i]
				self.menuItems[i].icon:SetTexture(items[i].icon)
				self.menuItems[i].text:SetText(items[i].caption)
				self.menuItems[i]:Show()
			else
				self.menuItems[i]:Hide()
			end
		end
		self:SetSelection(selection, true)
	end
	f.menu = CreateFrame("Frame", self:GetWidgetName(), f, "GameTooltipTemplate")
	f.menu:SetPoint("TOP", 0, -19)
	f.menu:SetScript("OnEnter", function(self) end)
	f.menu:Hide()
	f:SetItems(items, selection)
	return f	
end

ui.CreateText = function(self, text, parent, width, height, top, left, template, align)
	local f = CreateFrame("Frame",self:GetWidgetName(),parent)
	f:SetPoint("TOPLEFT",left,-top)
	f:SetSize(width,height)
	f.text = f:CreateFontString(nil, "ARTWORK", template or "GameTooltipText")
	f.text:SetJustifyH(align or "LEFT")
    f.text:SetAllPoints()
	f.text:SetText(text)
	f.SetText = function(self, text)
		self.text:SetText(text)
	end
	return f
end

ui.CreateEditBox = function(self, parent, width, height, top, left, default)
	local editbox = CreateFrame("EditBox",self:GetWidgetName(),parent,"InputBoxTemplate");
	editbox:SetPoint("TOPLEFT",left,-top)
	editbox:SetSize(width,height)
	editbox:SetText(default)
	editbox:SetAutoFocus(false)
	return editbox
end

ui.CreateCheckbox = function(self, parent, top, left, text, onChange)
	local cb = CreateFrame("CheckButton", self:GetWidgetName(), parent, "ChatConfigCheckButtonTemplate")
	cb:SetPoint("TOPLEFT",left,-top)
	local caption = _G[cb:GetName().."Text"]
	cb:SetHitRectInsets(0,caption:GetWidth(),0,0)
	caption:SetText(text)
	cb.onChange = onChange
	cb:SetScript("OnClick", function(self)
		self:onChange(self:GetChecked())
	end)
	return cb
end

ui.CreateTable = function(self, parent, top, left, height, width, dataSource, columns, selectMode)
	
	local tableObj = CreateFrame("ScrollFrame", nil, parent)
	tableObj:SetPoint("TOPLEFT",left,-top)
	tableObj.capacity = math.floor(height/16) - 1
	tableObj.columns = columns
	tableObj.position = 1
	tableObj.selectMode = selectMode or "none" -- none, single, multiple
	tableObj:SetSize(width, tableObj.capacity*16)
	
	local scrollbar = CreateFrame("Slider", self:GetWidgetName(), tableObj, "UIPanelScrollBarTemplate")
	scrollbar:SetPoint("RIGHT", tableObj,"RIGHT",0,-16)
	scrollbar:SetSize(16,16*tableObj.capacity-32)
	scrollbar:SetMinMaxValues(0, 1000)
	scrollbar:SetValue(0)
	
	tableObj.Sort = function(self, colIndex)
		if colIndex then
			local p = {}
			local colToSort = self.columns[math.abs(colIndex)].sortBy or self.columns[math.abs(colIndex)].name
			
			local temp = {}
			
			for indexx, item in pairs(self.dataSource) do
				temp[indexx] = {["idx"] = indexx, val = item[colToSort]}
			end
			if colIndex > 0 then
				table.sort(temp, function(v1, v2) 
					if not v1 and not v2 then return false end
					if not v1 then return false end
					if not v2 then return true end
					if not v1.val and not v2.val then return false end
					if not v1.val then return false end
					if not v2.val then return true end
					return v1.val < v2.val
				end)
			else
				table.sort(temp, function(v1, v2) 
					if not v1 and not v2 then return false end
					if not v1 then return false end
					if not v2 then return true end
					if not v1.val and not v2.val then return false end
					if not v1.val then return false end
					if not v2.val then return true end
					return v1.val > v2.val
				end)
			end
			
			for index = 1,#temp do
				p[index] = temp[index].idx
			end
			
			self.permutation = p
			self.colIndex = colIndex
		else
			local p = {}
			for i = 1,#self.dataSource do
				p[i] = i
			end
			self.permutation = p
			self.colIndex = nil
		end
	end
	
	tableObj.GetDataSource = function(self)
		return self.dataSource
	end
	
	tableObj.Update = function(self, dataSource)
		if dataSource then
			self.dataSource = dataSource
			self.lastPickedData = nil
			self:Sort(self.colIndex)
		end
		self.count = #self.dataSource
		if self.count <= self.capacity then
			self.position = 1
			self.scrollbar:SetValue(0)
			self.scrollbar:Hide()
		else
			self.scrollbar:Show()
			if self.position > self.count-self.capacity+1 then
				self.position = self.count-self.capacity+1
			end
			self.scrollbar:SetValue(1000*(self.position-1)/(self.count-self.capacity))
		end
		local i = self.position
		local rowNumber = 1
		while i <= self.count and rowNumber <= self.capacity do
			-- position i
			local dataIndex = self.permutation[i]
			self.content.rows[rowNumber]:Show()
			
			if self.dataSource[dataIndex].selected then
				self.content.rows[rowNumber].bg:SetColorTexture(0.5, 0.5, 0.5, 1)
			else
				self.content.rows[rowNumber].bg:SetTexture(nil)
			end
			
			for colNumber, column in pairs(self.columns) do
				self.content.rows[rowNumber].dataIndex = dataIndex
				local data = self.dataSource[dataIndex][column.name]
				local cell = self.content.rows[rowNumber].col[colNumber]
				if column.type then
					if column.type == "icon" then
						cell.bg:SetTexture(data)
					elseif column.type == "bool" then
						if data then
							cell.checkMark:SetTexture("INTERFACE\\BUTTONS\\UI-CheckBox-Check")
						else
							cell.checkMark:SetTexture()
						end
					elseif column.type == "action" then
						cell.text:SetText(data or column.caption)
					else
						cell.text:SetText(data)
					end
				else
					cell.text:SetText(data)
				end
			end
			i = i+1
			rowNumber = rowNumber + 1
		end
		while rowNumber <= self.capacity do
			self.content.rows[rowNumber].dataIndex = nil
			self.content.rows[rowNumber]:Hide()
			rowNumber = rowNumber + 1
		end
	end
	
	if tableObj.selectMode == "multiple" then
		tableObj.SetAllSelected = function(self, selected)
			for i = 1, #self.dataSource do
				self.dataSource[i].selected = selected
			end
			self.lastPickedData = #self.dataSource
			self:Update()
		end
	elseif tableObj.selectMode == "single" then
		tableObj.SetSelectedIndex = function(self, index)
			for i = 1, #self.dataSource do
				self.dataSource[i].selected = nil
			end
			if index and self.dataSource[index] then
				self.dataSource[index] = true
			end
			self:Update()
		end
		tableObj.GetSelectedIndex = function(self)
			for i = 1, #self.dataSource do
				if self.dataSource[i].selected then
					return i
				end
				return nil
			end
		end
	end
	
	local scrollbg = scrollbar:CreateTexture(nil, "BACKGROUND")
	scrollbg:SetAllPoints(scrollbar)
	scrollbg:SetColorTexture(0, 0, 0, 0.4)
	tableObj.scrollbar = scrollbar

	local content = CreateFrame("Frame", nil, tableObj)
	content:SetPoint("TOPLEFT",0,-16)
	content:SetSize(width-16, 16*tableObj.capacity)
	local contentbg = content:CreateTexture(nil, "BACKGROUND")
	contentbg:SetAllPoints(content)
	contentbg:SetColorTexture(0, 0, 0, 0.4)
	tableObj.content = content

	tableObj.header = {}
	
	local offset = 0
	
	for index, column in pairs(columns) do
		local h = CreateFrame("Frame", nil, tableObj)
		h:SetSize(column.width, 16)
		h:SetPoint("TOPLEFT", offset, 0)
		h.text = h:CreateFontString(nil, "ARTWORK", "GameFontWhite")
		h.text:SetJustifyH("LEFT")
		h.text:SetPoint("CENTER")
		h.text:SetSize(column.width-4,16)
		h.text:SetText(column.caption)
		h.index = index
		
		h.bg = h:CreateTexture(nil, "BACKGROUND")
		h.bg:SetAllPoints(h)
		h.bg:SetColorTexture(0, 0, 0, 0.8)
		
		h:SetScript("OnMouseUp", function(self, button)
			if h:GetParent().colIndex == self.index then
				h:GetParent():Sort(-self.index)
			else
				h:GetParent():Sort(self.index)
			end
			h:GetParent():Update()
		end)
		
		offset = offset + column.width
		tableObj.header[index] = h
	end
	
	content.rows = {};
	
	for i=1,tableObj.capacity do
		local row = CreateFrame("Frame", nil, content)
		row:SetSize(width-16, 16)
		row:SetPoint("TOPLEFT",0,(1-i)*16)
		row.bg = row:CreateTexture(nil, "BACKGROUND")
		row.bg:SetAllPoints(row)
		local offset = 0
		row.col = {}
		for colIndex, col in pairs(columns) do
			local c = CreateFrame("Frame", nil, row)
			c:SetSize(col.width, 16)
			c:SetPoint("TOPLEFT", offset, 0)
			if col.type and col.type == "icon" then
				c.bg = c:CreateTexture(nil, "BACKGROUND")
				c.bg:SetAllPoints(c)
			elseif col.type and col.type == "bool" then
				c.bg = c:CreateTexture(nil, "ARTWORK")
				c.bg:SetPoint("CENTER")
				c.bg:SetSize(16, 16)
				c.bg:SetTexture("INTERFACE\\BUTTONS\\UI-CheckBox-Up")
				c.bg:SetTexCoord(0.125, 0.875, 0.125, 0.875) -- l r t b
				c.bg:SetAlpha(1)
				c.checkMark = c:CreateTexture(nil, "OVERLAY")
				c.checkMark:SetPoint("CENTER")
				c.checkMark:SetSize(16, 16)
				c.checkMark:SetAlpha(1)
				c.checkMark:SetTexCoord(0.125, 0.875, 0.125, 0.875) -- l r t b
			else
				--c.text = c:CreateFontString(nil, "ARTWORK", "GameFontWhite")
				c.text = c:CreateFontString(nil, "ARTWORK", "NumberFontNormal")
				c.text:SetJustifyH(col.align or "LEFT")
				c.text:SetPoint("CENTER")
				c.text:SetSize(col.width-4,16)
				if col.type and col.type == "link" then
					c.text:SetTextColor(0.5,0.5,1)
				elseif col.type and col.type == "action" then
					c.text:SetTextColor(1.0,0.75,0.25)
				end			
			end
			
			if tableObj.selectMode == "multiple" then
				c:SetScript("OnMouseUp", function(self, button) 
					local dataIndex = self:GetParent().dataIndex
					if dataIndex then
						local tableObj = self:GetParent():GetParent():GetParent()
						if IsShiftKeyDown() and tableObj.lastPickedData then
							local thisIndex, lastIndex, minIndex, maxIndex
							for displayRow = 1, #tableObj.permutation do
								local dataRow = tableObj.permutation[displayRow]
								if tableObj.dataSource[dataRow].selected then
									minIndex = minIndex or displayRow
									maxIndex = displayRow
								end
								if dataRow == dataIndex then
									thisIndex = displayRow
								elseif dataRow == tableObj.lastPickedData then
									lastIndex = displayRow
								end	
							end
							local startIndex, endIndex
							if lastIndex == minIndex then
								lastIndex = maxIndex
							else
								lastIndex = minIndex
							end
							for i = 1, math.min(thisIndex, lastIndex)-1 do
								tableObj.dataSource[tableObj.permutation[i]].selected = nil
							end
							for i = math.min(thisIndex, lastIndex), math.max(thisIndex, lastIndex) do
								tableObj.dataSource[tableObj.permutation[i]].selected = true
							end
							for i = math.max(thisIndex, lastIndex)+1, #tableObj.permutation do
								tableObj.dataSource[tableObj.permutation[i]].selected = nil
							end
							tableObj.lastPickedData = dataIndex
							tableObj:Update()
						elseif IsControlKeyDown() then
							tableObj.dataSource[dataIndex].selected = not tableObj.dataSource[dataIndex].selected
							tableObj.lastPickedData = nil
							tableObj:Update()
						elseif not col.onMouseUp or IsShiftKeyDown() then
							for i = 1, #tableObj.dataSource do
								tableObj.dataSource[i].selected = nil
							end
							tableObj.dataSource[dataIndex].selected = true
							tableObj.lastPickedData = dataIndex
							tableObj:Update()
						else
							col.onMouseUp(tableObj, dataIndex)
						end
					end
				end)
			elseif tableObj.selectMode == "single" then
				c:SetScript("OnMouseUp", function(self, button) 
					local dataIndex = self:GetParent().dataIndex
					if dataIndex then
						if col.onMouseUp and not IsModifierKeyDown() then
							col.onMouseUp(self:GetParent():GetParent():GetParent(), dataIndex)
						else
							local tableObj = self:GetParent():GetParent():GetParent()
							tableObj:SetSelectedIndex(dataIndex)
						end
					end
				end)
			elseif col.onMouseUp then
				c:SetScript("OnMouseUp", function(self, button) 
					local dataIndex = self:GetParent().dataIndex
					if dataIndex then
						col.onMouseUp(self:GetParent():GetParent():GetParent(), dataIndex)
					end
				end)
			end
			
			if col.onEnter then
				c:SetScript("OnEnter", function(self) 
					if self:GetParent().dataIndex then
						col.onEnter(self:GetParent():GetParent():GetParent(), self:GetParent().dataIndex)
					end
				end)
			end
			if col.onLeave then
				c:SetScript("OnLeave", function(self) 
					if self:GetParent().dataIndex then
						col.onLeave(self:GetParent():GetParent():GetParent(), self:GetParent().dataIndex)
					end
				end)
			end
			row.col[colIndex] = c
			offset = offset + col.width
		end
		row:SetScript("OnMouseWheel", function(self, delta)
			local s = self:GetParent():GetParent()
			s.position = s.position-delta --*s.capacity
			if s.position > s.count-s.capacity+1 then
				s.position = s.count-s.capacity+1
			end
			if s.position < 1 then
				s.position = 1
			end
			s.scrollbar:SetValue(1000*(s.position-1)/(s.count-s.capacity))
			s:Update()
		end)
		content.rows[i] = row
	end
	
	scrollbar:SetScript("OnValueChanged", function (self, value)
		local f = self:GetParent()
		if f.count > f.capacity then
			local newPos = (value/1000)*(f.count-f.capacity)+1
			local newPosInt = math.floor(newPos)
			if newPos-newPosInt > 0.5 then
				newPosInt = newPosInt + 1
			end
			f.position = newPosInt
		else
			f.position = 1
		end
		f:Update()
	end)
	
	tableObj:Update(dataSource)
	
	return tableObj
end

ui.CreateList = function(self, parent, top, left, capacity, initFunc)

	local dummy = initFunc()
	local w,h = dummy:GetSize()
	
	local scrollframe = CreateFrame("ScrollFrame", nil, parent)
	if type(capacity) == "number" then
		scrollframe.capacityH = capacity
		scrollframe.capacityW = 1
	else
		scrollframe.capacityH = capacity[1]
		scrollframe.capacityW = capacity[2] or 1
	end
	
	scrollframe:SetPoint("TOPLEFT", left, -top)
	scrollframe:SetSize(w*scrollframe.capacityW + 16, h*scrollframe.capacityH)

	local scrollbar = CreateFrame("Slider", self:GetWidgetName(), scrollframe, "UIPanelScrollBarTemplate")
	scrollbar:SetPoint("TOPLEFT", w*scrollframe.capacityW,-16)
	scrollbar:SetSize(16, h*scrollframe.capacityH-32)
	scrollbar:SetMinMaxValues(0, 1000)
	scrollbar:SetValue(0)
	
	scrollframe.Update = function(self, numItems)
		if numItems then
			self.count = numItems
			local lineCount = math.ceil(self.count/self.capacityW)
			if lineCount <= self.capacityH then
				self.lineOffset = 1
				self.scrollbar:Hide()
			else
				if self.lineOffset > lineCount-self.capacityH+1 then
					self.lineOffset = lineCount-self.capacityH+1
				end
				self.scrollbar:SetValue(1000*(self.lineOffset-1)/(lineCount-self.capacityH))
				self.scrollbar:Show()
			end
		end
		local listIndex = (self.lineOffset-1) * self.capacityW + 1
		local frameIndex = 1
		while listIndex <= self.count and frameIndex <= self.capacityH * self.capacityW do
			self.content.items[frameIndex]:Update(listIndex)
			self.content.items[frameIndex]:Show()
			listIndex = listIndex+1
			frameIndex = frameIndex+1
		end
		while frameIndex <= self.capacityH * self.capacityW do
			self.content.items[frameIndex]:Hide()
			frameIndex = frameIndex+1
		end
	end
	
	scrollframe.ScrollTo = function(self, index)
		if index < (self.lineOffset-1) * self.capacityW + 1 or index > (self.lineOffset+self.capacityH-1) * self.capacityW then
			local lineCount = math.ceil(self.count/self.capacityW)
			self.lineOffset = math.max(math.min(lineCount-self.capacityH+1, math.ceil(index/self.capacityW)), 1)
			if lineCount > self.capacityH then
				self.scrollbar:SetValue(1000*(self.lineOffset-1)/(lineCount-self.capacityH))
			end
		end
		self:Update()
	end
	
	local scrollbg = scrollbar:CreateTexture(nil, "BACKGROUND")
	scrollbg:SetAllPoints(scrollbar)
	scrollbg:SetColorTexture(0, 0, 0, 0.4)
	scrollframe.scrollbar = scrollbar

	local content = CreateFrame("Frame", nil, scrollframe)
	content:SetPoint("TOPLEFT",0,0)
	content:SetSize(w*scrollframe.capacityW, h*scrollframe.capacityH)
	scrollframe.content = content

	content.items = {}
	
	local itemFrame
	for row = 1, scrollframe.capacityH do
		for col = 1, scrollframe.capacityW do
			if row == 1 and col == 1 then
				itemFrame = dummy
			else
				itemFrame = initFunc()
			end
			content.items[(row-1)*scrollframe.capacityW + col] = itemFrame
			itemFrame:SetParent(content)
			itemFrame:SetPoint("TOPLEFT",w*(col-1), h*(1-row))
			itemFrame:SetScript("OnMouseWheel", function(self, delta)
				local s = self:GetParent():GetParent()
				local lineCount = math.ceil(s.count/s.capacityW)
				s.lineOffset = math.max(math.min(lineCount-s.capacityH+1, s.lineOffset-delta), 1)
				if lineCount > s.capacityH then
					s.scrollbar:SetValue(1000*(s.lineOffset-1)/(lineCount-s.capacityH))
				end
				s:Update()
			end)
		end
	end
	
	scrollframe.count = 0
	scrollframe.lineOffset = 1
	scrollbar:SetScript("OnValueChanged", function (self, value)
		local f = self:GetParent()
		local lineCount = math.ceil(f.count/f.capacityW)
		if lineCount > f.capacityH then
			local newPos = (value/1000)*(lineCount-f.capacityH)+1
			local newPosInt = math.floor(newPos)
			if newPos-newPosInt > 0.5 then
				newPosInt = newPosInt + 1
			end
			f.lineOffset = newPosInt
		else
			f.lineOffset = 1
		end
		f:Update()
	end)
	
	scrollframe:Update(0)
	
	return scrollframe
end